import textwrap

import pytest

from conan.api.model import RecipeReference
from conan.errors import ConanException
from conan.internal.model.conf import ConfDefinition


@pytest.fixture()
def conf_definition():
    text = textwrap.dedent("""\
        tools.build:verbosity=quiet
        user.company.toolchain:flags=someflags
    """)
    c = ConfDefinition()
    c.loads(text)
    return c, text


def test_conf_definition(conf_definition):
    c, text = conf_definition
    # Round trip
    assert c.dumps() == text
    # access
    assert c.get("tools.build:verbosity") == "quiet"
    assert c.get("user.company.toolchain:flags") == "someflags"
    assert c.get("user.microsoft.msbuild:nonexist") is None
    assert c.get("user:nonexist") is None
    # bool
    assert bool(c)
    assert not bool(ConfDefinition())


@pytest.mark.parametrize("conf_name", [
    "tools.doesnotexist:never",
    "tools:doesnotexist",
    "core.doesnotexist:never",
    "core:doesnotexist"
])
def test_conf_definition_error(conf_definition, conf_name):
    c, text = conf_definition
    with pytest.raises(ConanException):
        c.get(conf_name)


def test_conf_update(conf_definition):
    c, _ = conf_definition
    text = textwrap.dedent("""\
        user.company.toolchain:flags=newvalue
        user.something:key=value
    """)
    c2 = ConfDefinition()
    c2.loads(text)
    c.update_conf_definition(c2)
    result = textwrap.dedent("""\
        tools.build:verbosity=quiet
        user.company.toolchain:flags=newvalue
        user.something:key=value
    """)
    assert c.dumps() == result


def test_conf_rebase(conf_definition):
    c, _ = conf_definition
    text = textwrap.dedent("""\
       user.company.toolchain:flags=newvalue
       tools.build:verbosity=verbose""")
    c2 = ConfDefinition()
    c2.loads(text)
    c.rebase_conf_definition(c2)
    # The c profile will have precedence, and "
    result = textwrap.dedent("""\
        tools.build:verbosity=quiet
        user.company.toolchain:flags=someflags
    """)
    assert c.dumps() == result


def test_conf_rebase_patterns():
    text = textwrap.dedent("""\
        user.company:conf1=value1
        user.company:conf2=[1, 2, 3]
        &:user.company:conf1=value2
        &:user.company:conf2+=[4, 5]
        """)
    global_conf = ConfDefinition()
    global_conf.loads(text)
    text = textwrap.dedent("""\
        user.company:conf1=newvalue1
        user.company:conf2=[6, 7]
        """)
    profile = ConfDefinition()
    profile.loads(text)
    profile.rebase_conf_definition(global_conf)
    # TODO: at the moment dumps() doesn't print += append
    result = textwrap.dedent("""\
        user.company:conf1=newvalue1
        user.company:conf2=[6, 7]
        &:user.company:conf1=value2
        &:user.company:conf2=[4, 5]
    """)
    assert profile.dumps() == result

    text = textwrap.dedent("""\
        &:user.company:conf1=newvalue1
        &:user.company:conf2=[6, 7]
        """)
    profile = ConfDefinition()
    profile.loads(text)
    profile.rebase_conf_definition(global_conf)
    result = textwrap.dedent("""\
        user.company:conf1=value1
        user.company:conf2=[1, 2, 3]
        &:user.company:conf1=newvalue1
        &:user.company:conf2=[6, 7]
       """)
    assert profile.dumps() == result

    text = "user.company:conf2+=[6, 7]"
    profile = ConfDefinition()
    profile.loads(text)
    profile.rebase_conf_definition(global_conf)
    result = textwrap.dedent("""\
        user.company:conf1=value1
        user.company:conf2=[1, 2, 3, 6, 7]
        &:user.company:conf1=value2
        &:user.company:conf2=[4, 5]
        """)
    assert profile.dumps() == result

    text = "&:user.company:conf2=+[6, 7]"
    profile = ConfDefinition()
    profile.loads(text)
    profile.rebase_conf_definition(global_conf)
    result = textwrap.dedent("""\
        user.company:conf1=value1
        user.company:conf2=[1, 2, 3]
        &:user.company:conf1=value2
        &:user.company:conf2=[6, 7, 4, 5]
        """)
    assert profile.dumps() == result


def test_conf_error_per_package():
    text = "*:core:non_interactive=minimal"
    c = ConfDefinition()
    with pytest.raises(ConanException,
                       match=r"Conf '\*:core:non_interactive' cannot have a package pattern"):
        c.loads(text)


def test_conf_error_uppercase():
    text = "tools.something:Verbosity=minimal"
    c = ConfDefinition()
    with pytest.raises(ConanException, match=r"Conf 'tools.something:Verbosity' must be lowercase"):
        c.loads(text)
    text = "tools.Something:verbosity=minimal"
    c = ConfDefinition()
    with pytest.raises(ConanException,
                       match=r"Conf 'tools.Something:verbosity' must be lowercase"):
        c.loads(text)


def test_parse_spaces():
    text = "core:non_interactive = minimal"
    c = ConfDefinition()
    c.loads(text)
    assert c.get("core:non_interactive") == "minimal"


@pytest.mark.parametrize("text, expected", [
    ("user.company.cpu:jobs=!", None),
    ("user.company.cpu:jobs=10", 10),
    ("user.company.build:ccflags=--m superflag", "--m superflag"),
    ("zlib:user.company.check:shared=True", True),
    ("zlib:user.company.check:shared_str='True'", "'True'"),
    ("user.company.list:objs=[1, 2, 3, 4, 'mystr', {'a': 1}]", [1, 2, 3, 4, 'mystr', {'a': 1}]),
    ("user.company.network:proxies={'url': 'http://api.site.com/api', 'dataType': 'json', 'method': 'GET'}",
     {'url': 'http://api.site.com/api', 'dataType': 'json', 'method': 'GET'})
])
def test_conf_get_different_type_input_objects(text, expected):
    """
    Testing any possible Python-evaluable-input-format introduced
    by the user
    """
    c = ConfDefinition()
    c.loads(text)
    assert c.get(text.split("=")[0]) == expected


@pytest.mark.parametrize("text1, text2, expected", [
    ("user.company.list:objs=[2, 3]", "user.company.list:objs=+[0, 1]", [0, 1, 2, 3]),
    ("user.company.list:objs=[2, 3]", "user.company.list:objs+=[4, 5]", [2, 3, 4, 5]),
    ("user.company.list:objs=[2, 3]", "user.company.list:objs+={'a': 1}", [2, 3, {'a': 1}]),
    ("user.company.list:objs=[2, 3]", "user.company.list:objs=+start", ["start", 2, 3]),
    ("user.company.list:objs=[2, 3]", "user.company.list:objs=[0, 1]", [0, 1]),
])
def test_conf_list_operations(text1, text2, expected):
    c1 = ConfDefinition()
    c1.loads(text1)
    c2 = ConfDefinition()
    c2.loads(text2)
    c1.update_conf_definition(c2)
    assert c1.get(text1.split("=")[0]) == expected


@pytest.mark.parametrize("text1, text2", [
    ("user.company.list:objs=value", "user.company.list:objs=['value']"),
    ("user.company.list:objs='value'", "user.company.list:objs=+[0, 1]"),
    ("user.company.list:objs={'a': 1}", "user.company.list:objs+={'b': 1}"),
    ("user.company.list:objs=True", "user.company.list:objs+=False"),
    ("user.company.list:objs=10", "user.company.list:objs=+11")
])
def test_conf_list_operations_fails_with_wrong_types(text1, text2):
    c1 = ConfDefinition()
    c1.loads(text1)
    c1_value_type = type(c1.get("user.company.list:objs")).__name__
    c2 = ConfDefinition()
    c2.loads(text2)
    with pytest.raises(ConanException) as exc_info:
        c1.update_conf_definition(c2)
    assert "It's not possible to compose list values and %s ones" % c1_value_type \
           in str(exc_info.value)


def test_compose_conf_complex():
    """
    Testing the composition between several ConfDefiniton objects and with
    different value types
    """
    text = textwrap.dedent("""\
        user.company.cpu:jobs=10
        user.company.build:ccflags=--m superflag
        zlib:user.company.check:shared=True
        zlib:user.company.check:shared_str="True"
        user.company.list:objs=[1, 2, 3, 4, 'mystr', {'a': 1}]
        user.company.network:proxies={'url': 'http://api.site.com/api', 'dataType': 'json', 'method': 'GET'}
    """)
    c = ConfDefinition()
    c.loads(text)
    text = textwrap.dedent("""\
        user.company.cpu:jobs=5
        user.company.build:ccflags=--m otherflag
        zlib:user.company.check:shared=!
        zlib:user.company.check:shared_str="False"
        user.company.list:objs+=[5, 6]
        user.company.list:objs=+0
        user.company.list:objs+={'b': 2}
        user.company.network:proxies={'url': 'http://api.site.com/apiv2'}
        """)
    c2 = ConfDefinition()
    c2.loads(text)
    c.update_conf_definition(c2)
    expected_text = textwrap.dedent("""\
        user.company.build:ccflags=--m otherflag
        user.company.cpu:jobs=5
        user.company.list:objs=[0, 1, 2, 3, 4, 'mystr', {'a': 1}, 5, 6, {'b': 2}]
        user.company.network:proxies={'url': 'http://api.site.com/apiv2'}
        zlib:user.company.check:shared=!
        zlib:user.company.check:shared_str="False"
    """)

    assert c.dumps() == expected_text


def test_compose_conf_dict_updates():
    c = ConfDefinition()
    c.loads("user.company:mydict={'1': 'a'}\n"
            "user.company:mydict2={'1': 'a'}")
    c2 = ConfDefinition()
    c2.loads("user.company:mydict={'2': 'b'}\n"
             "user.company:mydict2*={'2': 'b'}")
    c.update_conf_definition(c2)
    assert c.dumps() == ("user.company:mydict={'2': 'b'}\n"
                         "user.company:mydict2={'1': 'a', '2': 'b'}\n")

def test_compose_conf_numbers():
    c = ConfDefinition()
    c.loads("user.version:value=8.1\n"
            "foo/*:user.version:value=10")
    assert c.get("user.version:value") == 8.1
    assert c.get_conanfile_conf(RecipeReference.loads("foo/1.0")).get("user.version:value") == 10


def test_conf_get_check_type_and_default():
    text = textwrap.dedent("""\
        user.company.cpu:jobs=5
        user.company.build:ccflags=--m otherflag
        user.company.list:objs=[0, 1, 2, 3, 4, 'mystr', {'a': 1}, 5, 6]
        user.company.network:proxies={'url': 'http://api.site.com/apiv2', 'dataType': 'json', 'method': 'GET'}
        zlib:user.company.check:shared=!
        zlib:user.company.check:shared_str="False"
        zlib:user.company.check:static_str=off
        user.company.list:newnames+=myname
        core.download:parallel=True
        user:bad_value_0=Fasle
        user:bad_value_1=ture
        user:bad_value_2=10
        user:bad_value_3='00'
    """)
    c = ConfDefinition()
    c.loads(text)
    assert c.get("user.company.cpu:jobs", check_type=int) == 5
    assert c.get("user.company.cpu:jobs", check_type=None) == 5
    assert c.get("user.company.cpu:jobs", check_type=str) == "5"  # smart conversion
    with pytest.raises(ConanException) as exc_info:
        c.get("user.company.cpu:jobs", check_type=list)
    assert "[conf] user.company.cpu:jobs must be a list-like object." in str(exc_info.value)
    # Check type does not affect to default value
    assert c.get("user.non:existing", default=0, check_type=dict) == 0
    assert c.get("zlib:user.company.check:shared") is None  # unset value
    assert c.get("zlib:user.company.check:shared", default=[]) == []  # returning default
    assert c.get("zlib:user.company.check:shared", default=[], check_type=list) == []  # not raising exception
    assert c.get("zlib:user.company.check:shared_str") == '"False"'
    assert c.get("zlib:user.company.check:shared_str", check_type=bool) is False  # smart conversion
    assert c.get("zlib:user.company.check:static_str") == "off"
    assert c.get("zlib:user.company.check:static_str", check_type=bool) is False  # smart conversion
    assert c.get("user.company.list:newnames") == ["myname"]  # Placeholder is removed
    with pytest.raises(ConanException) as exc_info:
        c.get("core.download:parallel", check_type=int)
    assert ("[conf] core.download:parallel must be a int-like object. "
            "The value 'True' introduced is a bool object") in str(exc_info.value)
    with pytest.raises(ConanException) as exc_info:
        c.get("user:bad_value_0", check_type=bool)
    assert "[conf] user:bad_value_0 must be a boolean-like object (true/false, 1/0, on/off) and value 'Fasle' does not match it." in str(exc_info.value)
    with pytest.raises(ConanException) as exc_info:
        c.get("user:bad_value_1", check_type=bool)
    assert "[conf] user:bad_value_1 must be a boolean-like object (true/false, 1/0, on/off) and value 'ture' does not match it." in str(exc_info.value)
    with pytest.raises(ConanException) as exc_info:
        c.get("user:bad_value_2", check_type=bool)
    assert "[conf] user:bad_value_2 must be a boolean-like object (true/false, 1/0, on/off) and value '10' does not match it." in str(exc_info.value)
    with pytest.raises(ConanException) as exc_info:
        c.get("user:bad_value_3", check_type=bool)
    assert "[conf] user:bad_value_3 must be a boolean-like object (true/false, 1/0, on/off) and value '\'00\'' does not match it." in str(exc_info.value)


def test_conf_pop():
    text = textwrap.dedent("""\
        user.company.cpu:jobs=5
        user.company.build:ccflags=--m otherflag
        user.company.list:objs=[0, 1, 2, 3, 4, 'mystr', {'a': 1}, 5, 6]
        user.company.network:proxies={'url': 'http://api.site.com/apiv2', 'dataType': 'json', 'method': 'GET'}
        zlib:user.company.check:shared=!
        zlib:user.company.check:shared_str="False"
    """)
    c = ConfDefinition()
    c.loads(text)

    assert c.pop("user.company.network:proxies") == {'url': 'http://api.site.com/apiv2', 'dataType': 'json', 'method': 'GET'}
    assert c.pop("user.microsoft.msbuild:missing") is None
    assert c.pop("user.microsoft.msbuild:missing", default="fake") == "fake"
    assert c.pop("zlib:user.company.check:shared_str") == '"False"'


def test_conf_choices():
    confs = textwrap.dedent("""
    user.category:option1=1
    user.category:option2=3""")
    c = ConfDefinition()
    c.loads(confs)
    assert c.get("user.category:option1", choices=[1, 2]) == 1
    with pytest.raises(ConanException):
        c.get("user.category:option2", choices=[1, 2])


def test_conf_choices_default():
    # Does not conflict with the default value
    confs = textwrap.dedent("""
        user.category:option1=1""")
    c = ConfDefinition()
    c.loads(confs)
    assert c.get("user.category:option1", choices=[1, 2], default=7) == 1
    assert c.get("user.category:option2", choices=[1, 2], default=7) == 7


@pytest.mark.parametrize("scope", ["", "pkg/1.0:"])
@pytest.mark.parametrize("conf", [
    "user.foo:bar=1",
    "user:bar=1"
])
def test_conf_scope_patterns_ok(scope, conf):
    final_conf = scope + conf
    c = ConfDefinition()
    c.loads(final_conf)
    # Check it doesn't raise
    c.validate()


@pytest.mark.parametrize("conf", ["user.foo.bar=1"])
@pytest.mark.parametrize("scope, assert_message", [
    ("", "User conf 'user.foo.bar' invalid format, not 'user.org.group:conf'"),
    ("pkg/1.0:", "'pkg/1.0:user.foo.bar' does not exist in configuration list"),
])
def test_conf_scope_patterns_bad(scope, conf, assert_message):
    final_conf = scope + conf
    c = ConfDefinition()
    with pytest.raises(ConanException) as exc_info:
        c.loads(final_conf)
        c.validate()
    assert assert_message in str(exc_info.value)


@pytest.mark.parametrize("choices", [None, ["Foo", "Bar"]])
def test_unset_basic_same_behaviour(choices):
    c = ConfDefinition()
    assert c.get("user.company.cpu:jobs", choices=choices) is None

    c2 = ConfDefinition()
    c2.loads("user.company.cpu:jobs=!")
    assert c2.get("user.company.cpu:jobs", choices=choices) is None

    c3 = ConfDefinition()
    c3.loads("user.company.cpu:jobs=4")

    c4 = ConfDefinition()
    c4.loads("user.company.cpu:jobs=!")
    c3.update_conf_definition(c4)

    assert c3.get("user.company.cpu:jobs", choices=choices) is None



import os
import re
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient


@pytest.mark.parametrize("cmd", ["install", "build"])
def test_cmake_layout_build_folder(cmd):
    """ testing the tools.cmake.cmake_layout:build_folder config for
    both build and install commands
    """
    c = TestClient()
    abs_build = temp_folder()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            def layout(self):
                cmake_layout(self, generator="Ninja")  # Ninja so same in all OSs, single-config
        """)

    c.save({"conanfile.py": conanfile})
    c.run(f'{cmd} . -c tools.cmake.cmake_layout:build_folder="{abs_build}"')
    assert os.path.exists(os.path.join(abs_build, "Release", "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "build"))

    # Make sure that a non-existing folder will not fail and will be created
    new_folder = os.path.join(temp_folder(), "my build")
    c.run(f'{cmd} . -c tools.cmake.cmake_layout:build_folder="{new_folder}"')
    assert os.path.exists(os.path.join(new_folder, "Release", "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "build"))

    # Just in case we check that local build folder would be created if no arg is provided
    c.run('build . ')
    assert os.path.exists(os.path.join(c.current_folder, "build"))


@pytest.mark.parametrize("cmd", ["install", "build"])
def test_cmake_layout_build_folder_relative(cmd):
    """ Same as above, but with a relative folder, which is relative to the conanfile
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            def layout(self):
                cmake_layout(self, generator="Ninja")  # Ninja so same in all OSs, single-config
        """)

    c.save({"pkg/conanfile.py": conanfile})
    c.run(f'{cmd} pkg -c tools.cmake.cmake_layout:build_folder=mybuild')
    abs_build = os.path.join(c.current_folder, "pkg", "mybuild")
    assert os.path.exists(os.path.join(abs_build, "Release", "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "pkg", "build"))

    # relative path, pointing to a sibling folder
    c.run(f'{cmd} pkg -c tools.cmake.cmake_layout:build_folder=../mybuild')
    abs_build = os.path.join(c.current_folder, "mybuild")
    assert os.path.exists(os.path.join(abs_build, "Release", "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "build"))

    # Just in case we check that local build folder would be created if no arg is provided
    c.run('build pkg ')
    assert os.path.exists(os.path.join(c.current_folder, "pkg", "build"))


def test_test_cmake_layout_build_folder_test_package():
    """ relocate the test_package temporary build folders to elsewhere
    """
    c = TestClient()
    abs_build = temp_folder()
    test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def layout(self):
                cmake_layout(self, generator="Ninja")

            def test(self):
                pass
        """)

    c.save({"conanfile.py": GenConanfile("pkg", "0.1"),
            "test_package/conanfile.py": test_conanfile})
    c.run(f'create . -c tools.cmake.cmake_layout:test_folder="{abs_build}" '
          '-c tools.cmake.cmake_layout:build_folder_vars=[]')
    # Even if build_folder_vars=[] the "Release" folder is added always
    assert os.path.exists(os.path.join(abs_build, "Release", "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "test_package", "build"))

    c.run(f'create . -c tools.cmake.cmake_layout:build_folder_vars=[]')
    assert os.path.exists(os.path.join(c.current_folder, "test_package", "build"))


def test_test_cmake_layout_build_folder_test_package_temp():
    """ using always the same test_package build_folder will cause collisions.
    We need a mechanism to relocate, still provide unique folders for each build
    """
    c = TestClient()
    test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def layout(self):
                cmake_layout(self)

            def test(self):
                pass
        """)

    profile = textwrap.dedent("""
        include(default)
        [conf]
        tools.cmake.cmake_layout:test_folder=$TMP
        tools.cmake.cmake_layout:build_folder_vars=[]
        """)
    c.save({"conanfile.py": GenConanfile("pkg", "0.1"),
            "test_package/conanfile.py": test_conanfile,
            "profile": profile})
    c.run(f'create . -pr=profile')
    build_folder = re.search(r"Test package build folder: (\S+)", str(c.out)).group(1)
    assert os.path.exists(os.path.join(build_folder, "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "test_package", "build"))

    c.run(f'test test_package pkg/0.1 -pr=profile')
    build_folder = re.search(r"Test package build folder: (\S+)", str(c.out)).group(1)
    assert os.path.exists(os.path.join(build_folder, "generators", "conan_toolchain.cmake"))
    assert not os.path.exists(os.path.join(c.current_folder, "test_package", "build"))

    c.run(f'create . -c tools.cmake.cmake_layout:build_folder_vars=[]')
    assert os.path.exists(os.path.join(c.current_folder, "test_package", "build"))


def test_cmake_layout_build_folder_editable():
    """ testing how it works with editables. Layout
    code
      pkga  # editable add .
      pkgb  # editable add .
      app   # install -c build_folder="../../mybuild -c build_folder_vars="['self.name']"
        pkga-release-data.cmake
           pkga_PACKAGE_FOLDER_RELEASE = /abs/path/to/code/pkga
           pkga_INCLUDE_DIRS_RELEASE   = ${pkga_PACKAGE_FOLDER_RELEASE}/include
           pkga_LIB_DIRS_RELEASE =       /abs/path/to/mybuild/pkga/Release
    mybuild
      pkga/Release/
      pkgb/Release/
    """
    c = TestClient()
    base_folder = temp_folder()
    c.current_folder = os.path.join(base_folder, "code").replace("\\", "/")
    project_build_folder = os.path.join(base_folder, "mybuild").replace("\\", "/")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class Pkg(ConanFile):
            name = "{name}"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            def layout(self):
                cmake_layout(self, generator="Ninja")  # Ninja so same in all OSs, single-config
        """)

    c.save({"pkga/conanfile.py": conanfile.format(name="pkga"),
            "pkgb/conanfile.py": conanfile.format(name="pkgb"),
            "app/conanfile.py": GenConanfile().with_requires("pkga/0.1", "pkgb/0.1")
                                              .with_settings("build_type")
                                              .with_generator("CMakeDeps")})

    c.run("editable add pkga")
    c.run("editable add pkgb")
    conf = f'-c tools.cmake.cmake_layout:build_folder="../../mybuild" ' \
           '-c tools.cmake.cmake_layout:build_folder_vars="[\'self.name\']"'
    c.run(f'install app {conf} --build=editable')

    data = c.load("app/pkga-release-data.cmake")

    # The package is the ``code/pkga`` folder
    assert f'pkga_PACKAGE_FOLDER_RELEASE "{c.current_folder}/pkga"' in data
    # This is an absolute path, not relative, as it is not inside the package
    assert f'set(pkga_LIB_DIRS_RELEASE "{project_build_folder}/pkga/Release' in data
    data = c.load("app/pkgb-release-data.cmake")
    assert f'set(pkgb_LIB_DIRS_RELEASE "{project_build_folder}/pkgb/Release' in data

    assert os.path.exists(os.path.join(project_build_folder, "pkga", "Release", "generators",
                                       "conan_toolchain.cmake"))
    assert os.path.exists(os.path.join(project_build_folder, "pkgb", "Release", "generators",
                                       "conan_toolchain.cmake"))


def test_cmake_layout_editable_output_folder():
    """ testing how it works with editables, but --output-folder
    code
      pkga  # editable add . --output-folder = ../mybuild
      pkgb  # editable add . --output-folder = ../mybuild
      app   # install  -c build_folder_vars="['self.name']"
        pkga-release-data.cmake
           pkga_PACKAGE_FOLDER_RELEASE = /abs/path/to/mybuild
           pkga_INCLUDE_DIRS_RELEASE   = /abs/path/to/code/pkga/include
           pkga_LIB_DIRS_RELEASE =       pkga_PACKAGE_FOLDER_RELEASE/build/pkga/Release
    mybuild
      build
        pkga/Release/
        pkgb/Release/
    """
    c = TestClient()
    base_folder = temp_folder()
    c.current_folder = os.path.join(base_folder, "code")
    project_build_folder = os.path.join(base_folder, "mybuild").replace("\\", "/")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class Pkg(ConanFile):
            name = "{name}"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            def layout(self):
                cmake_layout(self, generator="Ninja")  # Ninja so same in all OSs, single-config
        """)

    c.save({"pkga/conanfile.py": conanfile.format(name="pkga"),
            "pkgb/conanfile.py": conanfile.format(name="pkgb"),
            "app/conanfile.py": GenConanfile().with_requires("pkga/0.1", "pkgb/0.1")
           .with_settings("build_type")
           .with_generator("CMakeDeps")})

    c.run(f'editable add pkga --output-folder="../mybuild"')
    c.run(f'editable add pkgb --output-folder="../mybuild"')
    conf = f'-c tools.cmake.cmake_layout:build_folder_vars="[\'self.name\']"'
    c.run(f'install app {conf} --build=editable')

    data = c.load("app/pkga-release-data.cmake")
    assert f'set(pkga_PACKAGE_FOLDER_RELEASE "{project_build_folder}")' in data
    # Thse folders are relative to the package
    assert 'pkga_LIB_DIRS_RELEASE "${pkga_PACKAGE_FOLDER_RELEASE}/build/pkga/Release' in data
    data = c.load("app/pkgb-release-data.cmake")
    assert 'pkgb_LIB_DIRS_RELEASE "${pkgb_PACKAGE_FOLDER_RELEASE}/build/pkgb/Release' in data

    assert os.path.exists(os.path.join(project_build_folder, "build", "pkga", "Release",
                                       "generators", "conan_toolchain.cmake"))
    assert os.path.exists(os.path.join(project_build_folder, "build", "pkgb", "Release",
                                       "generators", "conan_toolchain.cmake"))



import json
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_lock_build_tool_requires():
    c = TestClient(light=True)
    c.save({"common/conanfile.py": GenConanfile("common", "1.0").with_settings("os"),
            "tool/conanfile.py": GenConanfile("tool", "1.0").with_settings("os")
                                                            .with_requires("common/1.0"),
            "lib/conanfile.py": GenConanfile("lib", "1.0").with_settings("os")
                                                          .with_requires("tool/1.0"),
            "consumer/conanfile.py":
                GenConanfile("consumer", "1.0").with_settings("os")
                                               .with_requires("lib/1.0")
                                               .with_build_requires("tool/1.0")})
    c.run("export common")
    c.run("export tool")
    c.run("export lib")
    # cross compile Linux->Windows
    c.run("lock create consumer/conanfile.py -s:h os=Linux -s:b os=Windows --build=*")
    c.run("install --tool-requires=tool/1.0 --build=missing --lockfile=consumer/conan.lock "
          "-s:h os=Linux -s:b os=Windows")
    c.assert_listed_binary({"tool/1.0": ("78ba71aef65089d6e3244756171f9f37d5a76223", "Build"),
                            "common/1.0": ("ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715", "Build")},
                           build=True)


def test_lock_buildrequires_create():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("tool", "0.1")})
    c.run("create .  --build-require --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#2d65f1b4af1ce59028f96adbfe7ed5a2" in lock["build_requires"][0]


def test_lock_buildrequires_export():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("tool", "0.1")})
    c.run("export . --build-require --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#2d65f1b4af1ce59028f96adbfe7ed5a2" in lock["build_requires"][0]


def test_lock_buildrequires_create_transitive():
    c = TestClient(light=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "tool/conanfile.py": GenConanfile("tool", "0.1").with_requires("dep/0.1")})
    c.run("create dep")
    c.run("create tool --build-require --lockfile-out=conan.lock")
    lock = json.loads(c.load("conan.lock"))
    assert "tool/0.1#e4f0da4d9097c4da0725ea25b8bf83c8" in lock["build_requires"][0]
    assert "dep/0.1#f8c2264d0b32a4c33f251fe2944bb642" in lock["build_requires"][1]


def test_lock_create_build_require_transitive():
    """ cross compiling from Windows to Linux
    """
    c = TestClient(light=True)
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Dep(ConanFile):
            name = "dep"
            settings = "os"
            def package_id(self):
                self.output.info(f"MYOS:{self.info.settings.os}!!")
                self.output.info(f"TARGET:{self.settings_target.os}!!")
            """)
    tool = textwrap.dedent("""
        from conan import ConanFile
        class Tool(ConanFile):
           name = "tool"
           version = "0.1"
           requires = "dep/[*]"
           settings = "os"
           def generate(self):
               self.output.info(f"MYOS-GEN:{self.info.settings.os}!!")
               self.output.info(f"TARGET-GEN:{self.settings_target.os}!!")
           def package_id(self):
               self.output.info(f"MYOS:{self.info.settings.os}!!")
               self.output.info(f"TARGET:{self.settings_target.os}!!")
           """)
    c.save({"dep/conanfile.py": dep,
            "tool/conanfile.py": tool})
    c.run("create dep --build-require --version=0.1 -s:b os=Windows -s:h os=Linux")
    assert "dep/0.1: MYOS:Windows!!" in c.out
    assert "dep/0.1: TARGET:Linux!!" in c.out

    # The lockfile should contain dep in build-requires
    c.run("lock create tool --build-require -s:b os=Windows -s:h os=Linux")
    assert "dep/0.1: MYOS:Windows!!" in c.out
    assert "dep/0.1: TARGET:Linux!!" in c.out
    lock = json.loads(c.load("tool/conan.lock"))
    assert "dep/0.1" in lock["build_requires"][0]

    # Now try to apply it in  graph info, even if a new 0.2 verion si there
    c.run("create dep --build-require --version=0.2 -s:b os=Windows -s:h os=Linux")
    assert "dep/0.2: MYOS:Windows!!" in c.out
    assert "dep/0.2: TARGET:Linux!!" in c.out

    c.run("graph info tool --build-require -s:b os=Windows -s:h os=Linux")
    c.assert_listed_require({"dep/0.1": "Cache"}, build=True)
    assert "dep/0.1: MYOS:Windows!!" in c.out
    assert "dep/0.1: TARGET:Linux!!" in c.out
    assert "conanfile.py (tool/0.1): MYOS:Windows!!" in c.out
    assert "conanfile.py (tool/0.1): TARGET:Linux!!" in c.out
    assert "context: build" in c.out
    assert "context: host" not in c.out

    c.run("install tool --build-require -s:b os=Windows -s:h os=Linux")
    c.assert_listed_require({"dep/0.1": "Cache"}, build=True)
    assert "dep/0.1: MYOS:Windows!!" in c.out
    assert "dep/0.1: TARGET:Linux!!" in c.out
    assert "conanfile.py (tool/0.1): MYOS:Windows!!" in c.out
    assert "conanfile.py (tool/0.1): TARGET:Linux!!" in c.out
    assert "conanfile.py (tool/0.1): MYOS-GEN:Windows!!" in c.out
    assert "conanfile.py (tool/0.1): TARGET-GEN:Linux!!" in c.out


class TestTransitiveBuildRequires:
    @pytest.fixture()
    def client(self):
        # https://github.com/conan-io/conan/issues/13899
        client = TestClient(light=True)
        client.save({"zlib/conanfile.py": GenConanfile("zlib", "1.0"),
                     "cmake/conanfile.py": GenConanfile("cmake", "1.0").with_requires("zlib/1.0"),
                     "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_build_requires("cmake/1.0"),
                     "consumer/conanfile.py": GenConanfile().with_requires("pkg/[>=1.0]"),
                     })

        client.run("export zlib")
        client.run("export cmake")
        client.run("export pkg")
        client.run("lock create consumer/conanfile.py -pr:b=default --build=* "
                   "--lockfile-out=conan.lock")
        return client

    def test_transitive_build_require(self, client):
        # This used to crash, not anymore with the fix
        client.run("install consumer/conanfile.py --build=missing --lockfile=conan.lock")
        assert "zlib/1.0: Created package" in client.out
        assert "cmake/1.0: Created package" in client.out
        assert "pkg/1.0: Created package" in client.out

    def test_transitive_build_require_intermediate(self, client):
        # This used to crash, not anymore with the fix
        client.run("install pkg/conanfile.py --build=missing --lockfile=conan.lock")
        assert "zlib/1.0: Created package" in client.out
        assert "cmake/1.0: Created package" in client.out



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.file_server import TestFileServer
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save, set_dirty


class TestDownloadCache:

    def test_download_skip(self):
        """ basic proof that enabling download_cache avoids downloading things again
        """
        client = TestClient(default_server_user=True)
        # generate large random package file
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import save
            class Pkg(ConanFile):
                def package(self):
                    fileSizeInBytes = 11000000
                    with open(os.path.join(self.package_folder, "data.txt"), 'wb') as fout:
                        fout.write(os.urandom(fileSizeInBytes))
                """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=mypkg --version=0.1 --user=user --channel=testing")
        client.run("upload * --confirm -r default")
        client.run("remove * -c")

        # enable cache
        tmp_folder = temp_folder()
        client.save_home({"global.conf": f"core.download:download_cache={tmp_folder}"})
        client.run("install --requires=mypkg/0.1@user/testing")
        assert "mypkg/0.1@user/testing: Downloading" in client.out

        client.run("remove * -c")
        client.run("install --requires=mypkg/0.1@user/testing")
        assert "mypkg/0.1@user/testing: Downloading" not in client.out
        assert "conan_package.tgz from download cache, instead of downloading it" in client.out
        # removing the config downloads things
        client.save_home({"global.conf": ""})
        client.run("remove * -c")
        client.run("install --requires=mypkg/0.1@user/testing")
        assert "mypkg/0.1@user/testing: Downloading" in client.out

        client.save_home({"global.conf": f"core.download:download_cache={tmp_folder}"})

        client.run("remove * -c")
        client.run("install --requires=mypkg/0.1@user/testing")
        assert "mypkg/0.1@user/testing: Downloading" not in client.out
        assert "conan_package.tgz from download cache, instead of downloading it" in client.out

    def test_dirty_download(self):
        # https://github.com/conan-io/conan/issues/8578
        client = TestClient(default_server_user=True)
        tmp_folder = temp_folder()
        client.save_home({"global.conf": f"core.download:download_cache={tmp_folder}"})

        client.save({"conanfile.py": GenConanfile().with_package_file("file.txt", "content")})
        client.run("create . --name=pkg --version=0.1")
        client.run("upload * -c -r default")
        client.run("remove * -c")
        client.run("install --requires=pkg/0.1@")

        # Make the cache dirty
        # The "c" internal folder must exist, it is the actual storage of blobs
        cache_folder = os.path.join(tmp_folder, "c")
        for f in os.listdir(cache_folder):
            # damage the file
            path = os.path.join(cache_folder, f)
            assert os.path.isfile(path)
            save(path, "broken!")
            set_dirty(path)

        client.run("remove * -c")
        client.run("install --requires=pkg/0.1@")
        assert "Downloading" in client.out

        client.run("remove * -c")
        client.run("install --requires=pkg/0.1@")
        # TODO  assert "Downloading" not in client.out

    def test_user_downloads_cached_newtools(self):
        client = TestClient()
        file_server = TestFileServer()
        client.servers["file_server"] = file_server
        save(os.path.join(file_server.store, "myfile.txt"), "some content")
        save(os.path.join(file_server.store, "myfile2.txt"), "some query")
        save(os.path.join(file_server.store, "myfile3.txt"), "some content 3")

        tmp_folder = temp_folder()
        client.save_home({"global.conf": f"core.sources:download_cache={tmp_folder}"})
        # badchecksums are not cached
        conanfile = textwrap.dedent("""
           from conan import ConanFile
           from conan.tools.files import download
           class Pkg(ConanFile):
               def source(self):
                   download(self, "%s/myfile.txt", "myfile.txt", md5="kk")
           """ % file_server.fake_url)
        client.save({"conanfile.py": conanfile})
        client.run("source .", assert_error=True)
        assert "ConanException: md5 signature failed for" in client.out
        assert "Provided signature: kk" in client.out

        # There are 2 things in the cache, not sha256, no caching
        assert 0 == len(os.listdir(tmp_folder))  # Nothing was cached

        # This is the right checksum
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import download
            class Pkg(ConanFile):
                def source(self):
                    md5 = "9893532233caff98cd083a116b013c0b"
                    md5_2 = "0dc8a17658b1c7cfa23657780742a353"
                    sha256 = "bcc23055e479c1050455f5bb457088cfae3cbb2783f7579a7df9e33ea9f43429"
                    download(self, "{0}/myfile.txt", "myfile.txt", md5=md5)
                    download(self, "{0}/myfile3.txt", "myfile3.txt", sha256=sha256)
                    download(self, "{0}/myfile.txt?q=myfile2.txt", "myfile2.txt", md5=md5_2)
            """).format(file_server.fake_url)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        assert "some content" in client.load("myfile.txt")
        assert "some query" in client.load("myfile2.txt")
        assert "some content 3" in client.load("myfile3.txt")

        # remove remote and local files
        os.remove(os.path.join(file_server.store, "myfile3.txt"))
        os.remove(os.path.join(client.current_folder, "myfile.txt"))
        os.remove(os.path.join(client.current_folder, "myfile2.txt"))
        os.remove(os.path.join(client.current_folder, "myfile3.txt"))
        # Will use the cached one
        client.run("source .")
        assert "some content" == client.load("myfile.txt")
        assert "some query" == client.load("myfile2.txt")
        assert "some content 3" in client.load("myfile3.txt")

        # disabling cache will make it fail
        client.save_home({"global.conf": ""})
        client.run("source .", assert_error=True)
        assert "ERROR: conanfile.py: Error in source() method, line 10" in client.out
        assert "Not found" in client.out

    def test_download_relative_error(self):
        """ relative paths are not allowed
        """
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": GenConanfile().with_package_file("file.txt", "content")})
        c.run("create . --name=mypkg --version=0.1 --user=user --channel=testing")
        c.run("upload * --confirm -r default")
        c.run("remove * -c")

        # enable cache
        c.save_home({"global.conf": f"core.download:download_cache=mytmp_folder"})
        c.run("install --requires=mypkg/0.1@user/testing", assert_error=True)
        assert 'core.download:download_cache must be an absolute path' in c.out



import textwrap

from conan.test.utils.tools import TestClient


def test_conan_version_str():
    """
    conanfile.version should always be a string.
    If comparison neeeded use Version(self.version) or self.ref.version
    """
    # https://github.com/conan-io/conan/issues/10372
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.scm import Version

        class Lib(ConanFile):
            name = "pkg"
            version = "1.0"

            def _assert_data(self):
                assert isinstance(self.version, str)
                assert not isinstance(self.version, Version)

            def configure(self):
                self._assert_data()

            def source(self):
                self._assert_data()
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")



import os
import platform
import textwrap

import pytest
from unittest.mock import patch

from conan import conan_version
from conan.internal.api.detect import detect_api
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save
from conan.test.utils.tools import TestClient


@pytest.fixture
def client():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            def generate(self):
                for k, v in self.conf.items():
                    self.output.info("{}${}".format(k, v))
        """)
    client.save({"conanfile.py": conanfile})
    return client


def test_basic_composition(client):
    profile1 = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:vs_version=Slow
        tools.cmake.cmaketoolchain:generator=Extra
        """)
    profile2 = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:max_cpu_count=High
        tools.meson.mesontoolchain:backend=Super
        """)
    client.save({"profile1": profile1,
                 "profile2": profile2})
    client.run("install . -pr=profile1")
    assert "tools.build:verbosity$quiet" in client.out
    assert "tools.microsoft.msbuild:vs_version$Slow" in client.out
    assert "tools.cmake.cmaketoolchain:generator$Extra" in client.out

    client.run("install . -pr=profile1 -pr=profile2")
    assert "tools.build:verbosity$quiet" in client.out
    assert "tools.microsoft.msbuild:vs_version$Slow" in client.out
    assert "tools.microsoft.msbuild:max_cpu_count$High" in client.out
    assert "tools.cmake.cmaketoolchain:generator$Extra" in client.out
    assert "tools.meson.mesontoolchain:backend$Super" in client.out

    client.run("install . -pr=profile2 -pr=profile1")
    assert "tools.build:verbosity$quiet" in client.out
    assert "tools.microsoft.msbuild:vs_version$Slow" in client.out
    assert "tools.microsoft.msbuild:max_cpu_count$High" in client.out
    assert "tools.cmake.cmaketoolchain:generator$Extra" in client.out
    assert "tools.meson.mesontoolchain:backend$Super" in client.out


def test_basic_inclusion(client):
    profile1 = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:vs_version=Slow
        tools.cmake.cmaketoolchain:generator=Extra
        """)
    profile2 = textwrap.dedent("""\
        include(profile1)
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:max_cpu_count=High
        tools.meson.mesontoolchain:backend=Super
        """)
    client.save({"profile1": profile1,
                 "profile2": profile2})

    client.run("install . -pr=profile2")
    assert "tools.build:verbosity$quiet" in client.out
    assert "tools.microsoft.msbuild:vs_version$Slow" in client.out
    assert "tools.microsoft.msbuild:max_cpu_count$High" in client.out
    assert "tools.cmake.cmaketoolchain:generator$Extra" in client.out
    assert "tools.meson.mesontoolchain:backend$Super" in client.out


def test_composition_conan_conf(client):
    conf = textwrap.dedent("""\
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:vs_version=Slow
        tools.cmake.cmaketoolchain:generator=Extra
        """)
    client.save_home({"global.conf": conf})
    profile = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:max_cpu_count=High
        tools.meson.mesontoolchain:backend=Super
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile")
    assert "tools.build:verbosity$quiet" in client.out
    assert "tools.microsoft.msbuild:vs_version$Slow" in client.out
    assert "tools.microsoft.msbuild:max_cpu_count$High" in client.out
    assert "tools.cmake.cmaketoolchain:generator$Extra" in client.out
    assert "tools.meson.mesontoolchain:backend$Super" in client.out


def test_core_global_conf_not_in_profile(client):
    client.save_home({"global.conf": "core.upload:retry=1"})
    profile = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile")
    assert "tools.build:verbosity$quiet" in client.out
    # The core conf is not shown
    assert "core.upload:retry=1" not in client.out

    # also with -cc command line argument
    client.run("install . -pr=profile -cc core.upload:retry=1")
    assert "tools.build:verbosity$quiet" in client.out
    # The core conf is not shown
    assert "core.upload:retry=1" not in client.out


def test_new_config_file(client):
    conf = textwrap.dedent("""\
        tools.build:verbosity=quiet
        user.mycompany.myhelper:myconfig=myvalue
        *:tools.cmake.cmaketoolchain:generator=X
        """)
    client.save_home({"global.conf": conf})
    client.run("install .")
    assert "tools.build:verbosity$quiet" in client.out
    assert "user.mycompany.myhelper:myconfig$myvalue" in client.out
    assert "tools.cmake.cmaketoolchain:generator$X" in client.out
    assert "read_only" not in client.out

    conf = textwrap.dedent("""\
            tools.build:verbosity=notice
            user.mycompany.myhelper:myconfig=myvalue
            *:tools.cmake.cmaketoolchain:generator=X
            cache:read_only=True
            """)
    client.save_home({"global.conf": conf})
    client.run("install .", assert_error=True)
    assert "[conf] 'cache:read_only' does not exist in configuration list" in client.out


@patch("conan.__version__", "1.26.0")
def test_new_config_file_required_version():
    client = TestClient()
    conf = textwrap.dedent("""\
        core:required_conan_version=>=2.0
        """)
    client.save_home({"global.conf": conf})
    client.run("install .", assert_error=True)
    assert ("Current Conan version (1.26.0) does not satisfy the defined one (>=2.0)"
            in client.out)


def test_composition_conan_conf_overwritten_by_cli_arg(client):
    conf = textwrap.dedent("""\
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:max_cpu_count=Slow
        """)
    client.save_home({"global.conf": conf})
    profile = textwrap.dedent("""\
        [conf]
        tools.build:verbosity=quiet
        tools.microsoft.msbuild:vs_version=High
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile -c tools.build:verbosity=verbose "
               "-c tools.meson.mesontoolchain:backend=Super")
    assert "tools.build:verbosity$verbose" in client.out
    assert "tools.microsoft.msbuild:max_cpu_count$Slow" in client.out
    assert "tools.microsoft.msbuild:vs_version$High" in client.out
    assert "tools.meson.mesontoolchain:backend$Super" in client.out


def test_composition_conan_conf_different_data_types_by_cli_arg(client):
    """
    Testing if you want to introduce a list/dict via cli

    >> conan install . -c "tools.build.flags:ccflags+=['-Werror']"
    >> conan install . -c "tools.microsoft.msbuildtoolchain:compile_options={'ExceptionHandling': 'Async'}"

    """
    conf = textwrap.dedent("""\
        tools.build:cflags=["-Wall"]
        """)
    client.save_home({"global.conf": conf})
    client.run('install . -c "tools.build:cflags+=[\'-Werror\']" '
               '-c "tools.microsoft.msbuildtoolchain:compile_options={\'ExceptionHandling\': \'Async\'}"')

    assert "tools.build:cflags$['-Wall', '-Werror']" in client.out
    assert "tools.microsoft.msbuildtoolchain:compile_options${'ExceptionHandling': 'Async'}" in client.out


def test_jinja_global_conf(client):
    client.save_home({"global.conf": "user.mycompany:parallel = {{os.cpu_count()/2}}\n"
                                     "user.mycompany:other = {{platform.system()}}\n"
                                     "user.mycompany:dist = {{distro.id() if distro else '42'}}\n"
                                     "user.conan:version = {{conan_version}}-{{conan_version>0.1}}"})
    client.run("install .")
    assert "user.mycompany:parallel={}".format(os.cpu_count()/2) in client.out
    assert "user.mycompany:other={}".format(platform.system()) in client.out
    assert f"user.conan:version={conan_version}-True" in client.out
    if platform.system() == "Linux":
        import distro
        assert "user.mycompany:dist={}".format(distro.id()) in client.out
    else:
        assert "user.mycompany:dist=42" in client.out


def test_jinja_global_conf_include(client):
    global_conf = textwrap.dedent("""\
        {% include "user_global.conf" %}
        {% import "user_global.conf" as vars %}
        user.mycompany:dist = {{vars.myvar*2}}
        """)
    user_global_conf = textwrap.dedent("""\
        {% set myvar = 42 %}
        user.mycompany:parallel = {{myvar}}
        """)
    client.save_home({"global.conf": global_conf})
    save(os.path.join(client.cache_folder, "user_global.conf"), user_global_conf)
    client.run("install .")
    assert "user.mycompany:parallel=42" in client.out
    assert "user.mycompany:dist=84" in client.out


def test_jinja_global_conf_paths():
    c = TestClient()
    global_conf = 'user.mycompany:myfile = {{os.path.join(conan_home_folder, "myfile")}}'
    c.save_home({"global.conf": global_conf})
    c.run("config show *")
    cache_folder = c.cache_folder.replace("\\", "/")
    assert f"user.mycompany:myfile: {os.path.join(cache_folder, 'myfile')}" in c.out


def test_profile_detect_os_arch():
    """ testing OS & ARCH just to test that detect_api is injected
    """
    c = TestClient()
    global_conf = textwrap.dedent("""
        user.myteam:myconf1={{detect_api.detect_os()}}
        user.myteam:myconf2={{detect_api.detect_arch()}}
        """)

    c.save_home({"global.conf": global_conf})
    c.run("config show *")
    _os = detect_api.detect_os()
    _arch = detect_api.detect_arch()
    assert f"user.myteam:myconf1: {_os}" in c.out
    assert f"user.myteam:myconf2: {_arch}" in c.out


def test_empty_conf_valid():
    tc = TestClient()
    profile = textwrap.dedent(r"""
    [conf]
    user:unset=
    """)
    conanfile = textwrap.dedent(r"""
    from conan import ConanFile

    class BasicConanfile(ConanFile):
        name = "pkg"
        version = "1.0"

        def generate(self):
            self.output.warning(f'My unset conf variable is: "{self.conf.get("user:unset")}"')
            self.output.warning(f'My unset conf is {"NOT" if self.conf.get("user:unset") == None else ""} set')
    """)
    tc.save({"conanfile.py": conanfile, "profile": profile})

    tc.run("create .")
    assert 'pkg/1.0: WARN: My unset conf is NOT set' in tc.out

    tc.run("create . -pr=profile")
    assert 'pkg/1.0: WARN: My unset conf variable is: ""' in tc.out
    assert 'pkg/1.0: WARN: My unset conf is  set' in tc.out

    tc.run("create . -c user:unset=")
    assert 'pkg/1.0: WARN: My unset conf variable is: ""' in tc.out
    assert 'pkg/1.0: WARN: My unset conf is  set' in tc.out

    tc.run('create . -c user:unset=""')
    assert 'pkg/1.0: WARN: My unset conf variable is: ""' in tc.out
    assert 'pkg/1.0: WARN: My unset conf is  set' in tc.out

    # And ensure this actually works for the normal case, just in case
    tc.run("create . -c user:unset=Hello")
    assert 'pkg/1.0: WARN: My unset conf variable is: "Hello"' in tc.out
    assert 'pkg/1.0: WARN: My unset conf is  set' in tc.out


def test_nonexisting_conf():
    c = TestClient()
    c.save({"conanfile.txt": ""})
    c.run("install . -c tools.unknown:conf=value", assert_error=True)
    assert "ERROR: [conf] 'tools.unknown:conf' does not exist in configuration" in c.out
    c.run("install . -c user.some:var=value")  # This doesn't fail
    c.run("install . -c user.some.var=value", assert_error=True)
    assert "ERROR: User conf 'user.some.var' invalid format, not 'user.org.group:conf'" in c.out
    c.run("install . -c tool.build:verbosity=v", assert_error=True)
    assert "ERROR: [conf] 'tool.build:verbosity' does not exist in configuration" in c.out


def test_nonexisting_conf_global_conf():
    c = TestClient()
    c.save_home({"global.conf":  "tools.unknown:conf=value"})
    c.save({"conanfile.txt": ""})
    c.run("install . ", assert_error=True)
    assert "ERROR: [conf] 'tools.unknown:conf' does not exist in configuration" in c.out


def test_global_conf_auto_created():
    c = TestClient()
    c.run("config list")  # all commands will trigger
    global_conf = c.load_home("global.conf")
    assert "# core:non_interactive = True" in global_conf


def test_command_line_core_conf():
    c = TestClient()
    c.run("config show * -cc core:default_profile=potato")
    assert "core:default_profile: potato" in c.out
    c.run("config show * -cc core:default_profile=potato -cc core:default_build_profile=orange")
    assert "core:default_profile: potato" in c.out
    assert "core:default_build_profile: orange" in c.out

    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("export .")

    tfolder = temp_folder()
    c.run(f'list * -cc core.cache:storage_path="{tfolder}"')
    assert "WARN: There are no matching recipe references" in c.out
    c.run(f'list *')
    assert "WARN: There are no matching recipe references" not in c.out
    assert "pkg/0.1" in c.out

    c.run("list * -cc user.xxx:yyy=zzz", assert_error=True)
    assert "ERROR: Only core. values are allowed in --core-conf. Got user.xxx:yyy=zzz" in c.out


def test_build_test_consumer_only():
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def generate(self):
                skip = self.conf.get("tools.build:skip_test", check_type=bool)
                self.output.info(f'SKIP-TEST: {skip}')
        """)
    pkg = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                requires = "dep/0.1"
                def generate(self):
                    self.output.info(f'SKIP-TEST: {self.conf.get("tools.build:skip_test")}')
            """)
    c.save_home({"global.conf": "tools.build:skip_test=True\n&:tools.build:skip_test=False"})
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": pkg,
            "pkg/test_package/conanfile.py": GenConanfile().with_test("pass")})
    c.run("create dep")
    assert "dep/0.1: SKIP-TEST: False" in c.out
    c.run('create pkg --build=* -tf=""')
    assert "dep/0.1: SKIP-TEST: True" in c.out
    assert "pkg/0.1: SKIP-TEST: False" in c.out
    c.run('create pkg --build=*')
    assert "dep/0.1: SKIP-TEST: True" in c.out
    assert "pkg/0.1: SKIP-TEST: False" in c.out
    c.run('install pkg --build=*')
    assert "dep/0.1: SKIP-TEST: True" in c.out
    assert "conanfile.py (pkg/0.1): SKIP-TEST: False" in c.out


def test_conf_should_be_immutable():
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def generate(self):
                self.conf.append("user.myteam:myconf", "value1")
                self.output.info(f'user.myteam:myconf: {self.conf.get("user.myteam:myconf")}')
        """)
    pkg = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            requires = "dep/0.1"
            def generate(self):
                self.output.info(f'user.myteam:myconf: {self.conf.get("user.myteam:myconf")}')
        """)
    c.save_home({"global.conf": 'user.myteam:myconf=["root_value"]'})
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": pkg})
    c.run("create dep")
    assert "dep/0.1: user.myteam:myconf: ['root_value', 'value1']" in c.out
    c.run('create pkg --build=*')
    assert "dep/0.1: user.myteam:myconf: ['root_value', 'value1']" in c.out
    # The pkg/0.1 output should be non-modified
    assert "pkg/0.1: user.myteam:myconf: ['root_value']" in c.out


def test_especial_strings_fail():
    # https://github.com/conan-io/conan/issues/15777
    c = TestClient()
    global_conf = textwrap.dedent("""
        user.mycompany:myfile = re
        user.mycompany:myother = fnmatch
        user.mycompany:myfunct = re.search
        user.mycompany:mydict = {1: 're', 2: 'fnmatch'}
        """)
    c.save_home({"global.conf": global_conf})
    c.run("config show *")
    assert "user.mycompany:myfile: re" in c.out
    assert "user.mycompany:myother: fnmatch" in c.out
    assert "user.mycompany:myfunct: re.search" in c.out
    assert "user.mycompany:mydict: {1: 're', 2: 'fnmatch'}" in c.out


def test_conf_test_package():
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def generate(self):
                self.output.info(f'user.myteam:myconf: {self.conf.get("user.myteam:myconf")}')
        """)
    pkg = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def requirements(self):
                self.requires(self.tested_reference_str)
            def generate(self):
                self.output.info(f'user.myteam:myconf: {self.conf.get("user.myteam:myconf")}')
            def test(self):
                pass
        """)
    c.save({"conanfile.py": dep,
            "test_package/conanfile.py": pkg})
    c.run("create . -c &:user.myteam:myconf=myvalue")
    assert "dep/0.1: user.myteam:myconf: myvalue" in c.out
    assert "dep/0.1 (test package): user.myteam:myconf: myvalue" in c.out



from conan.tools.build.flags import architecture_flag, architecture_link_flag, libcxx_flags, threads_flags
import os
import textwrap
from pathlib import Path

from conan.tools.env.virtualbuildenv import VirtualBuildEnv
from jinja2 import Template

from conan.tools.build.cross_building import cross_building
from conan.tools.files import save
from conan.tools.microsoft.visual import VCVars
from conan.tools.premake.premakedeps import PREMAKE_ROOT_FILE


def _generate_flags(self, conanfile):
    template = textwrap.dedent(
        """\
        {% if extra_cflags %}
        -- C flags retrieved from CFLAGS environment, conan.conf(tools.build:cflags), extra_cflags and compiler settings
        filter { files { "**.c" } }
            buildoptions { {{ extra_cflags }} }
        filter {}
        {% endif %}
        {% if extra_cxxflags %}
        -- CXX flags retrieved from CXXFLAGS environment, conan.conf(tools.build:cxxflags), extra_cxxflags and compiler settings
        filter { files { "**.cpp", "**.cxx", "**.cc" } }
            buildoptions { {{ extra_cxxflags }} }
        filter {}
        {% endif %}
        {% if extra_ldflags %}
        -- Link flags retrieved from LDFLAGS environment, conan.conf(tools.build:sharedlinkflags), conan.conf(tools.build:exelinkflags), extra_cxxflags and compiler settings
        linkoptions { {{ extra_ldflags }} }
        {% endif %}
        {% if extra_defines %}
        -- Defines retrieved from DEFINES environment, conan.conf(tools.build:defines) and extra_defines
        defines { {{ extra_defines }} }
        {% endif %}
    """
    )

    def format_list(items):
        return ", ".join(f'"{item}"' for item in items) if items else None

    def to_list(value):
        return value if isinstance(value, list) else [value] if value else []

    arch_flags = to_list(architecture_flag(self._conanfile))
    cxx_flags, libcxx_compile_definitions = libcxx_flags(self._conanfile)
    arch_link_flags = to_list(architecture_link_flag(self._conanfile))
    thread_flags_list = threads_flags(self._conanfile)

    extra_defines = format_list(
        conanfile.conf.get("tools.build:defines", default=[], check_type=list)
        + self.extra_defines
        + to_list(libcxx_compile_definitions)
    )
    extra_c_flags = format_list(
        conanfile.conf.get("tools.build:cflags", default=[], check_type=list)
        + self.extra_cflags
        + arch_flags
        + thread_flags_list
    )
    extra_cxx_flags = format_list(
        conanfile.conf.get("tools.build:cxxflags", default=[], check_type=list)
        + to_list(cxx_flags)
        + self.extra_cxxflags
        + arch_flags
        + thread_flags_list
    )
    extra_ld_flags = format_list(
        conanfile.conf.get("tools.build:sharedlinkflags", default=[], check_type=list)
        + conanfile.conf.get("tools.build:exelinkflags", default=[], check_type=list)
        + self.extra_ldflags
        + arch_flags
        + arch_link_flags
        + thread_flags_list
    )

    return (
        Template(template, trim_blocks=True, lstrip_blocks=True)
        .render(
            extra_defines=extra_defines,
            extra_cflags=extra_c_flags,
            extra_cxxflags=extra_cxx_flags,
            extra_ldflags=extra_ld_flags,
        )
        .strip()
    )


class _PremakeProject:
    _premake_project_template = textwrap.dedent(
        """\
    project "{{ name }}"
        {% if kind %}
        kind "{{ kind }}"
        {% endif %}
        {% if flags %}
    {{ flags | indent(indent_level, first=True) }}
        {% endif %}
    """
    )

    def __init__(self, name, conanfile) -> None:
        self.name = name
        self.kind = None
        self.extra_cxxflags = []
        self.extra_cflags = []
        self.extra_ldflags = []
        self.extra_defines = []
        self.disable = False
        self._conanfile = conanfile

    def _generate(self):
        """Generates project block"""
        flags_content = _generate_flags(self, self._conanfile)  # Generate flags specific to this project
        return Template(self._premake_project_template, trim_blocks=True, lstrip_blocks=True).render(
            name=self.name,
            kind="None" if self.disable else self.kind,
            flags=flags_content,
            indent_level=4,
        )


class PremakeToolchain:
    """
    PremakeToolchain generator
    """

    filename = "conantoolchain.premake5.lua"
    # Keep template indented correctly for Lua output
    _premake_file_template = textwrap.dedent(
        """\
    #!lua
    -- Conan auto-generated toolchain file
    {% if has_conan_deps %}
    -- Include conandeps.premake5.lua with Conan dependency setup
    include("conandeps.premake5.lua")
    {% endif %}

    -- Base build directory
    local locationDir = path.normalize("{{ build_folder }}")

    -- Generate workspace configurations
    for wks in premake.global.eachWorkspace() do
        workspace(wks.name)
            -- Set base location for all workspaces
            location(locationDir)
            targetdir(path.join(locationDir, "bin"))
            objdir(path.join(locationDir, "obj"))

            {% if cppstd %}
            cppdialect "{{ cppstd }}"
            {% endif %}
            {% if cstd %}
            cdialect "{{ cstd }}"
            {% endif %}
            {% if shared != None %}
            -- IMPORTANT: this global setting will only apply `project`s which do not have `kind` set.
            -- IMPORTANT: This will not override existing `kind` set in `project` block.
            -- To let conan take control over `kind` of the libraries, DO NOT SET `kind` (StaticLib or
            -- SharedLib) in `project` block.
            kind "{{ "SharedLib" if shared else "StaticLib" }}"
            {% endif %}
            {% if fpic != None %}
            -- Enable position independent code
            pic "{{ "On" if fpic else "Off" }}"
            {% endif %}
            filter { "architecture: not wasm64" }
                -- TODO: There is an issue with premake and "wasm64" when system is declared "emscripten"
                system "{{ target_build_os }}"
            filter {}
            {% if macho_to_amd64 %}
            -- TODO: this should be fixed by premake: https://github.com/premake/premake-core/issues/2136
            buildoptions "-arch x86_64"
            linkoptions "-arch x86_64"
            {% endif %}
            {% if target_build_os == "emscripten" %}
            filter { "system:emscripten", "kind:ConsoleApp or WindowedApp" }
                -- Replace built in .wasm extension to .js to generate also a JavaScript files
                targetextension ".js"
            filter {}
            {% endif %}
            {% if flags %}
    {{ flags | indent(indent_level, first=True) }}
            {% endif %}

            filter { "system:macosx" }
                -- SHARED LIBS
                -- In the future we could add an opt in configuration to run
                -- fix_apple_shared_install_name on executables to have a similar behavior as CMake
                -- generator. Premake does not allow adding absolute RCPATHS
                -- Due to this limitation, if a consumer depends on a premake shared recipe, it will
                -- require to run conanrun script to setup proper DYLD_LIBRARY_PATH
                -- Reference: https://github.com/premake/premake-core/issues/2262#issuecomment-2378250385
                linkoptions { "-Wl,-rpath,@loader_path" }
            filter {}

            conan_setup()
    end

        {% for project in projects.values() %}

    {{ project._generate() }}
        {% endfor %}
    """
    )

    def __init__(self, conanfile):
        """
        :param conanfile: ``< ConanFile object >`` The current recipe object. Always use ``self``.
        """
        self._conanfile = conanfile
        self._projects = {}
        # Extra flags
        #: List of extra ``CXX`` flags. Added to ``buildoptions``.
        self.extra_cxxflags = []
        #: List of extra ``C`` flags. Added to ``buildoptions``.
        self.extra_cflags = []
        #: List of extra linker flags. Added to ``linkoptions``.
        self.extra_ldflags = []
        #: List of extra preprocessor definitions. Added to ``defines``.
        self.extra_defines = []

    def project(self, project_name):
        """
        The returned object will also have the same properties as the workspace but will only affect
        the project with the name.
        :param project_name: The name of the project inside the workspace to be updated.
        :return: ``<PremakeProject>`` object which allow to set project specific flags.
        """
        if project_name not in self._projects:
            self._projects[project_name] = _PremakeProject(project_name, self._conanfile)
        return self._projects[project_name]

    def generate(self):
        """
        Creates a ``conantoolchain.premake5.lua`` file which will properly configure build paths,
        binary paths, configuration settings and compiler/linker flags based on toolchain
        configuration.
        """
        premake_conan_deps = Path(self._conanfile.generators_folder) / PREMAKE_ROOT_FILE
        cppstd = self._conanfile.settings.get_safe("compiler.cppstd")
        if cppstd:
            # See premake possible cppstd values: https://premake.github.io/docs/cppdialect/
            if cppstd.startswith("gnu"):
                cppstd = f"gnu++{cppstd[3:]}"
            elif cppstd[0].isnumeric():
                cppstd = f"c++{cppstd}"

        compilers_build_mapping = self._conanfile.conf.get(
            "tools.build:compiler_executables", default={}, check_type=dict
        )
        if compilers_build_mapping:
            build_env = VirtualBuildEnv(self._conanfile, auto_generate=False)
            env = build_env.environment()
            if "c" in compilers_build_mapping:
                env.define("CC", compilers_build_mapping["c"])
            if "cpp" in compilers_build_mapping:
                env.define("CXX", compilers_build_mapping["cpp"])
            build_env.generate()

        macho_to_amd64 = (
            self._conanfile.settings.arch
            if cross_building(self._conanfile) and self._conanfile.settings.os == "Macos"
            else None
        )

        content = Template(self._premake_file_template, trim_blocks=True, lstrip_blocks=True).render(
            # Pass posix path for better cross-platform compatibility in Lua
            build_folder=Path(self._conanfile.build_folder).as_posix(),
            has_conan_deps=premake_conan_deps.exists(),
            cppstd=cppstd,
            cstd=self._conanfile.settings.get_safe("compiler.cstd"),
            shared=self._conanfile.options.get_safe("shared"),
            fpic=self._conanfile.options.get_safe("fPIC"),
            target_build_os=self._target_build_os(),
            macho_to_amd64=macho_to_amd64,
            projects=self._projects,
            flags=_generate_flags(self, self._conanfile),
            indent_level=8,
        )
        save(
            self,
            os.path.join(self._conanfile.generators_folder, self.filename),
            content,
        )
        # Generate VCVars if using MSVC
        if "msvc" in self._conanfile.settings.compiler:
            VCVars(self._conanfile).generate()

    def _target_build_os(self):
        conan_os = str(self._conanfile.settings.os)
        if conan_os == "Macos":
            return "macosx"
        return conan_os.lower()



import os
import textwrap

import pytest

from conan.api.model import PkgReference
from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient, NO_SETTINGS_PACKAGE_ID


@pytest.fixture
def conanfile():
    conan_file = str(GenConanfile()
                     .with_import("import os")
                     .with_import("from conan.tools.files import copy, save").
                     with_require("base/1.0"))

    conan_file += """
    no_copy_sources = True

    def layout(self):
        self.folders.source = "my_sources"
        self.folders.build = "my_build"

    def source(self):
        self.output.warning("Source folder: {}".format(self.source_folder))
        # The layout describes where the sources are, not force them to be there
        save(self, "source.h", "foo")

    def build(self):
        self.output.warning("Build folder: {}".format(self.build_folder))
        save(self, "build.lib", "bar")

    def package(self):
        self.output.warning("Package folder: {}".format(self.package_folder))
        save(self, os.path.join(self.package_folder, "LICENSE"), "bar")
        copy(self, "*.h", self.source_folder, os.path.join(self.package_folder, "include"))
        copy(self, "*.lib", self.build_folder, os.path.join(self.package_folder, "lib"))

    def package_info(self):
        # This will be easier when the layout declares also the includedirs etc
        self.cpp_info.includedirs = ["include"]
        self.cpp_info.libdirs = ["lib"]
    """
    return conan_file


def test_create_test_package_no_layout():
    """The test package using the new generators work (having the generated files in the build
    folder)"""
    client = TestClient()
    conanfile_test = textwrap.dedent("""
        import os

        from conan import ConanFile, tools

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeDeps", "CMakeToolchain"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def build(self):
                assert os.path.exists("conan_toolchain.cmake")
                self.output.warning("hey! building")
                self.output.warning(os.getcwd())

            def test(self):
                self.output.warning("hey! testing")
    """)
    client.save({"conanfile.py": GenConanfile(), "test_package/conanfile.py": conanfile_test})
    client.run("create . --name=lib --version=1.0")
    assert "hey! building" in client.out
    assert "hey! testing" in client.out


def test_create_test_package_with_layout():
    """The test package using the new generators work (having the generated files in the build
    folder)"""
    client = TestClient()
    conanfile_test = textwrap.dedent("""
        import os

        from conan import ConanFile, tools
        from conan.tools.cmake import CMakeToolchain, CMake, CMakeDeps

        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def generate(self):
                deps = CMakeDeps(self)
                deps.generate()
                tc = CMakeToolchain(self)
                tc.generate()

            def layout(self):
                self.folders.generators = "my_generators"

            def build(self):
                assert os.path.exists("my_generators/conan_toolchain.cmake")
                self.output.warning("hey! building")
                self.output.warning(os.getcwd())

            def test(self):
                self.output.warning("hey! testing")
    """)
    client.save({"conanfile.py": GenConanfile(), "test_package/conanfile.py": conanfile_test})
    client.run("create . --name=lib --version=1.0")
    assert "hey! building" in client.out
    assert "hey! testing" in client.out


def test_cache_in_layout(conanfile):
    """The layout in the cache is used too, always relative to the "base" folders that the cache
    requires. But by the default, the "package" is not followed
    """
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=base --version=1.0")

    client.save({"conanfile.py": conanfile})
    client.run("create . --name=lib --version=1.0")
    pkg_layout = client.created_layout()
    sf = client.exported_layout().source()
    bf = pkg_layout.build()
    pf = pkg_layout.package()

    source_folder = os.path.join(sf, "my_sources")
    build_folder = os.path.join(bf, "my_build")

    # Check folders match with the declared by the layout
    assert "Source folder: {}".format(source_folder) in client.out
    assert "Build folder: {}".format(build_folder) in client.out
    # Check the source folder
    assert os.path.exists(os.path.join(source_folder, "source.h"))

    # Check the build folder
    assert os.path.exists(os.path.join(build_folder, "build.lib"))

    # Check the conaninfo
    assert os.path.exists(os.path.join(pf, "conaninfo.txt"))


def test_same_conanfile_local(conanfile):
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=base --version=1.0")

    client.save({"conanfile.py": conanfile})

    source_folder = os.path.join(client.current_folder, "my_sources")
    build_folder = os.path.join(client.current_folder, "install", "my_build")

    client.run("install . --name=lib --version=1.0 -of=install")
    client.run("source .")
    assert "Source folder: {}".format(source_folder) in client.out
    assert os.path.exists(os.path.join(source_folder, "source.h"))

    client.run("build .  -of=install")
    assert "Build folder: {}".format(build_folder) in client.out
    assert os.path.exists(os.path.join(build_folder, "build.lib"))


def test_cpp_package():
    client = TestClient()

    conan_hello = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Pkg(ConanFile):
            def package(self):
                save(self, os.path.join(self.package_folder, "foo/include/foo.h"), "")
                save(self, os.path.join(self.package_folder,"foo/libs/foo.lib"), "")

            def layout(self):
                self.cpp.package.includedirs = ["foo/include"]
                self.cpp.package.libdirs = ["foo/libs"]
                self.cpp.package.libs = ["foo"]
             """)

    client.save({"conanfile.py": conan_hello})
    client.run("create . --name=hello --version=1.0")
    rrev = client.exported_recipe_revision()
    ref = RecipeReference.loads("hello/1.0")
    ref.revision = rrev
    pref = PkgReference(ref, NO_SETTINGS_PACKAGE_ID)
    package_folder = client.get_latest_pkg_layout(pref).package().replace("\\", "/") + "/"

    conan_consumer = textwrap.dedent("""
        from conan import ConanFile
        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "hello/1.0"
            generators = "CMakeDeps"
            def generate(self):
                info = self.dependencies["hello"].cpp_info
                self.output.warning("**includedirs:{}**".format(info.includedirs))
                self.output.warning("**libdirs:{}**".format(info.libdirs))
                self.output.warning("**libs:{}**".format(info.libs))
        """)

    client.save({"conanfile.py": conan_consumer})
    client.run("install .")
    out = str(client.out).replace(r"\\", "/").replace(package_folder, "")
    assert "**includedirs:['foo/include']**" in out
    assert "**libdirs:['foo/libs']**" in out
    assert "**libs:['foo']**" in out
    host_arch = client.get_default_host_profile().settings['arch']
    cmake = client.load(f"hello-release-{host_arch}-data.cmake")

    assert 'set(hello_INCLUDE_DIRS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/foo/include")' in cmake
    assert 'set(hello_LIB_DIRS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/foo/libs")' in cmake
    assert 'set(hello_LIBS_RELEASE foo)' in cmake


def test_git_clone_with_source_layout():
    client = TestClient()
    repo = temp_folder()
    conanfile = textwrap.dedent("""
           import os
           from conan import ConanFile
           class Pkg(ConanFile):
               exports_sources = "*.txt"

               def layout(self):
                   self.folders.source = "src"

               def source(self):
                   self.run('git clone "{}" .')
       """).format(repo.replace("\\", "/"))

    client.save({"conanfile.py": conanfile,
                 "myfile.txt": "My file is copied"})
    with client.chdir(repo):
        client.save({"cloned.txt": "foo"}, repo)
        client.init_git_repo()

    client.run("create . --name=hello --version=1.0")
    sf = client.exported_layout().source()
    assert os.path.exists(os.path.join(sf, "myfile.txt"))
    # The conanfile is cleared from the root before cloning
    assert not os.path.exists(os.path.join(sf, "conanfile.py"))
    assert not os.path.exists(os.path.join(sf, "cloned.txt"))

    assert os.path.exists(os.path.join(sf, "src", "cloned.txt"))
    assert not os.path.exists(os.path.join(sf, "src", "myfile.txt"))



import platform
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def bazel_output_root_dir():
    return temp_folder(path_with_spaces=False).replace("\\", "/")


@pytest.fixture(scope="module")
def bazelrc():
    return textwrap.dedent("""
        build:Debug -c dbg --copt=-g
        build:Release -c opt
        build:RelWithDebInfo -c opt --copt=-O3 --copt=-DNDEBUG
        build:MinSizeRel  -c opt --copt=-Os --copt=-DNDEBUG
        build --color=yes
        build:withTimeStamps --show_timestamps
        """)


@pytest.fixture(scope="module")
def base_profile():
    return textwrap.dedent("""
        include(default)
        [settings]
        build_type={build_type}

        [conf]
        tools.google.bazel:bazelrc_path=["{curdir}/mybazelrc"]
        tools.google.bazel:configs=["{build_type}", "withTimeStamps"]
        """)


@pytest.mark.parametrize("build_type", ["Debug", "Release", "RelWithDebInfo", "MinSizeRel"])
@pytest.mark.tool("bazel", "6.x")
def test_basic_exe_6x(bazelrc, build_type, base_profile, bazel_output_root_dir):
    client = TestClient(path_with_spaces=False)
    client.run(f"new bazel_exe -d name=myapp -d version=1.0 -d output_root_dir={bazel_output_root_dir}")
    # The build:<config> define several configurations that can be activated by passing
    # the bazel config with tools.google.bazel:configs
    client.save({"mybazelrc": bazelrc})
    profile = base_profile.format(build_type=build_type,
                                  curdir=client.current_folder.replace("\\", "/"))
    client.save({"my_profile": profile})
    client.run("create . --profile=./my_profile")
    if build_type != "Debug":
        assert "myapp/1.0: Hello World Release!" in client.out
    else:
        assert "myapp/1.0: Hello World Debug!" in client.out


@pytest.mark.parametrize("build_type", ["Debug", "Release", "RelWithDebInfo", "MinSizeRel"])
@pytest.mark.tool("bazel", "7.x")
def test_basic_exe(bazelrc, build_type, base_profile, bazel_output_root_dir):
    client = TestClient(path_with_spaces=False)
    client.run(f"new bazel_7_exe -d name=myapp -d version=1.0 -d output_root_dir={bazel_output_root_dir}")
    # The build:<config> define several configurations that can be activated by passing
    # the bazel config with tools.google.bazel:configs
    client.save({"mybazelrc": bazelrc})
    profile = base_profile.format(build_type=build_type,
                                  curdir=client.current_folder.replace("\\", "/"))
    client.save({"my_profile": profile})
    client.run("create . --profile=./my_profile")
    if build_type != "Debug":
        assert "myapp/1.0: Hello World Release!" in client.out
    else:
        assert "myapp/1.0: Hello World Debug!" in client.out


@pytest.mark.tool("bazel", "8.x")
def test_basic_lib(bazelrc, base_profile, bazel_output_root_dir):
    """
    Issue related: https://github.com/conan-io/conan/issues/17438
    """
    client = TestClient(path_with_spaces=False)
    client.run(f"new bazel_7_lib -d name=mylib -d version=1.0 -d output_root_dir={bazel_output_root_dir}")
    client.run("create .")
    assert "mylib/1.0: Hello World Release!" in client.out


@pytest.mark.parametrize("shared", [False, True])
@pytest.mark.tool("bazel", "6.x")
def test_transitive_libs_consuming_6x(shared, bazel_output_root_dir):
    """
    Testing the next dependencies structure for shared/static libs

    /.
    |- myfirstlib/
         |- conanfile.py
         |- WORKSPACE
         |- main/
            |- BUILD
            |- myfirstlib.cpp
            |- myfirstlib.h
    |- mysecondlib/  (requires myfirstlib)
         |- conanfile.py
         |- WORKSPACE
         |- main/
            |- BUILD
            |- mysecondlib.cpp
            |- mysecondlib.h
         |- test_package/
            |- WORKSPACE
            |- conanfile.py
            |- main
                |- example.cpp
                |- BUILD
    """
    client = TestClient(path_with_spaces=False)
    # A regular library made with Bazel
    with client.chdir("myfirstlib"):
        client.run(f"new bazel_lib -d name=myfirstlib -d version=1.2.11 -d output_root_dir={bazel_output_root_dir}")
        conanfile = client.load("conanfile.py")
        conanfile += """
        self.cpp_info.defines.append("MY_DEFINE=\\"MY_VALUE\\"")
        self.cpp_info.defines.append("MY_OTHER_DEFINE=2")
        if self.settings.os != "Windows":
            self.cpp_info.system_libs.append("m")
        else:
            self.cpp_info.system_libs.append("ws2_32")
        """
        client.save({"conanfile.py": conanfile})
        client.run(f"create . -o '*:shared={shared}' -tf ''")  # skipping tests

    with client.chdir("mysecondlib"):
        # We prepare a consumer with Bazel (library mysecondlib using myfirstlib)
        # and a test_package with an example executable
        os_ = platform.system()
        client.run(f"new bazel_lib -d name=mysecondlib -d version=1.0 -d output_root_dir={bazel_output_root_dir}")
        conanfile = client.load("conanfile.py")
        conanfile = conanfile.replace('generators = "BazelToolchain"',
                                      'generators = "BazelToolchain", "BazelDeps"\n'
                                      '    requires = "myfirstlib/1.2.11"')
        workspace = textwrap.dedent("""
        load("@//conan:dependencies.bzl", "load_conan_dependencies")
        load_conan_dependencies()
        """)
        bazel_build_linux = textwrap.dedent("""\
        cc_library(
            name = "mysecondlib",
            srcs = ["mysecondlib.cpp"],
            hdrs = ["mysecondlib.h"],
            deps = [ "@myfirstlib//:myfirstlib" ]
        )
        """)
        bazel_build = textwrap.dedent("""\
        cc_library(
            name = "mysecondlib",
            srcs = ["mysecondlib.cpp"],
            hdrs = ["mysecondlib.h"],
            deps = [ "@myfirstlib//:myfirstlib" ]
        )

        cc_shared_library(
            name = "mysecondlib_shared",
            shared_lib_name = "libmysecondlib_shared.{}",
            deps = [":mysecondlib"],
        )
        """.format("dylib" if os_ == "Darwin" else "dll"))
        mysecondlib_cpp = textwrap.dedent("""
        #include <iostream>
        #include "mysecondlib.h"
        #include "myfirstlib.h"
        #include <cmath>

        void mysecondlib(){
            std::cout << "mysecondlib() First define " << MY_DEFINE << " and other define " << MY_OTHER_DEFINE << std::endl;
            myfirstlib();
            // This comes from the systemlibs declared in the myfirstlib
            sqrt(25);
        }
        void mysecondlib_print_vector(const std::vector<std::string> &strings) {
            for(std::vector<std::string>::const_iterator it = strings.begin(); it != strings.end(); ++it) {
                std::cout << "mysecondlib/1.0 " << *it << std::endl;
            }
        }
        """)
        mysecondlib_cpp_win = textwrap.dedent("""
        #include <iostream>
        #include "mysecondlib.h"
        #include "myfirstlib.h"
        #include <WinSock2.h>

        void mysecondlib(){
            SOCKET foo; // From the system library
            std::cout << "mysecondlib() First define " << MY_DEFINE << " and other define " << MY_OTHER_DEFINE << std::endl;
            myfirstlib();
        }
        void mysecondlib_print_vector(const std::vector<std::string> &strings) {
            for(std::vector<std::string>::const_iterator it = strings.begin(); it != strings.end(); ++it) {
                std::cout << "mysecondlib/1.0 " << *it << std::endl;
            }
        }
        """)
        # Overwriting files
        client.save({"conanfile.py": conanfile,
                     "WORKSPACE": workspace,
                     "main/BUILD": bazel_build_linux if os_ == "Linux" else bazel_build,
                     "main/mysecondlib.cpp": mysecondlib_cpp if os_ != "Windows" else mysecondlib_cpp_win,
                     })

        client.run(f"create . -o '*:shared={shared}'")
        assert "mysecondlib() First define MY_VALUE and other define 2" in client.out
        assert "myfirstlib/1.2.11: Hello World Release!"


@pytest.mark.parametrize("shared", [False, True])
@pytest.mark.tool("bazel", "7.x")
@pytest.mark.skipif(platform.system() == "Linux",
                    reason="Conan CI fails (likely related to parallel "
                           "tests running??). Skipping it for now!")
def test_transitive_libs_consuming_7x(shared, bazel_output_root_dir):
    """
    Testing the next dependencies structure for shared/static libs

    /.
    |- myfirstlib/
         |- conanfile.py
         |- MODULE.bazel
         |- main/
            |- BUILD
            |- myfirstlib.cpp
            |- myfirstlib.h
    |- mysecondlib/  (requires myfirstlib)
         |- conanfile.py
         |- MODULE.bazel
         |- main/
            |- BUILD
            |- mysecondlib.cpp
            |- mysecondlib.h
         |- test_package/
            |- MODULE.bazel
            |- conanfile.py
            |- main
                |- example.cpp
                |- BUILD
    """
    client = TestClient(path_with_spaces=False)
    # A regular library made with Bazel
    with client.chdir("myfirstlib"):
        client.run(f"new bazel_7_lib -d name=myfirstlib -d version=1.2.11 -d output_root_dir={bazel_output_root_dir}")
        conanfile = client.load("conanfile.py")
        conanfile += """
        self.cpp_info.defines.append("MY_DEFINE=\\"MY_VALUE\\"")
        self.cpp_info.defines.append("MY_OTHER_DEFINE=2")
        if self.settings.os != "Windows":
            self.cpp_info.system_libs.append("m")
        else:
            self.cpp_info.system_libs.append("ws2_32")
        # Issue: https://github.com/conan-io/conan/issues/18748
        from conan.tools.apple.apple import is_apple_os
        if is_apple_os(self):
            self.cpp_info.frameworks = ["CoreFoundation"]
        """
        client.save({"conanfile.py": conanfile})
        client.run(f"create . -o '*:shared={shared}' -tf ''")  # skipping tests

    with client.chdir("mysecondlib"):
        # We prepare a consumer with Bazel (library mysecondlib using myfirstlib)
        # and a test_package with an example executable
        os_ = platform.system()
        client.run(f"new bazel_7_lib -d name=mysecondlib -d version=1.0 -d output_root_dir={bazel_output_root_dir}")
        conanfile = client.load("conanfile.py")
        conanfile = conanfile.replace('generators = "BazelToolchain"',
                                      'generators = "BazelToolchain", "BazelDeps"\n'
                                      '    requires = "myfirstlib/1.2.11"')
        workspace = textwrap.dedent("""
        load_conan_dependencies = use_extension("//conan:conan_deps_module_extension.bzl", "conan_extension")
        use_repo(load_conan_dependencies, "myfirstlib")
        """)
        bazel_build_linux = textwrap.dedent("""\
        cc_library(
            name = "mysecondlib",
            srcs = ["mysecondlib.cpp"],
            hdrs = ["mysecondlib.h"],
            deps = [ "@myfirstlib//:myfirstlib" ]
        )
        """)
        bazel_build = textwrap.dedent("""\
        cc_library(
            name = "mysecondlib",
            srcs = ["mysecondlib.cpp"],
            hdrs = ["mysecondlib.h"],
            deps = [ "@myfirstlib//:myfirstlib" ]
        )

        cc_shared_library(
            name = "mysecondlib_shared",
            shared_lib_name = "libmysecondlib_shared.{}",
            deps = [":mysecondlib"],
        )
        """.format("dylib" if os_ == "Darwin" else "dll"))
        mysecondlib_cpp = textwrap.dedent("""
        #include <iostream>
        #include "mysecondlib.h"
        #include "myfirstlib.h"
        #include <cmath>

        void mysecondlib(){
            std::cout << "mysecondlib() First define " << MY_DEFINE << " and other define " << MY_OTHER_DEFINE << std::endl;
            myfirstlib();
            // This comes from the systemlibs declared in the myfirstlib
            sqrt(25);
        }
        void mysecondlib_print_vector(const std::vector<std::string> &strings) {
            for(std::vector<std::string>::const_iterator it = strings.begin(); it != strings.end(); ++it) {
                std::cout << "mysecondlib/1.0 " << *it << std::endl;
            }
        }
        """)
        mysecondlib_cpp_win = textwrap.dedent("""
        #include <iostream>
        #include "mysecondlib.h"
        #include "myfirstlib.h"
        #include <WinSock2.h>

        void mysecondlib(){
            SOCKET foo; // From the system library
            std::cout << "mysecondlib() First define " << MY_DEFINE << " and other define " << MY_OTHER_DEFINE << std::endl;
            myfirstlib();
        }
        void mysecondlib_print_vector(const std::vector<std::string> &strings) {
            for(std::vector<std::string>::const_iterator it = strings.begin(); it != strings.end(); ++it) {
                std::cout << "mysecondlib/1.0 " << *it << std::endl;
            }
        }
        """)
        # Overwriting files
        client.save({"conanfile.py": conanfile,
                     "MODULE.bazel": workspace,
                     "main/BUILD": bazel_build_linux if os_ == "Linux" else bazel_build,
                     "main/mysecondlib.cpp": mysecondlib_cpp if os_ != "Windows" else mysecondlib_cpp_win,
                     })

        client.run(f"create . -o '*:shared={shared}'")
        assert "mysecondlib() First define MY_VALUE and other define 2" in client.out
        assert "myfirstlib/1.2.11: Hello World Release!"


@pytest.mark.tool("bazel", "8.x")
def test_empty_bazel_query():
    """
    Test that following a simple steps using the BazelDeps and running
    a global `bazel query //...` runs OK (bazel >= 8.0)

    Issue related: https://github.com/conan-io/conan/issues/18743
    """
    zlib = GenConanfile("zlib", "0.1")
    consumer = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.google import BazelDeps, bazel_layout

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        requires = "zlib/0.1"

        def layout(self):
            bazel_layout(self)

        def generate(self):
            bz = BazelDeps(self)
            bz.generate()
    """)
    module = textwrap.dedent("""\
    load_conan_dependencies = use_extension("//conan:conan_deps_module_extension.bzl", "conan_extension")
    use_repo(load_conan_dependencies, "zlib")
    """)
    client = TestClient()
    client.save({
        "zlib/conanfile.py": zlib,
        "consumer/conanfile.py": consumer,
        "consumer/MODULE.bazel": module,
    })
    client.run("create zlib")
    client.run("install consumer")
    with client.chdir("consumer"):
        client.run_command("bazel query //...")
    assert "//conan/zlib:zlib" in client.out
    assert "//conan/zlib:zlib_binaries" in client.out



import textwrap

from conan.test.utils.tools import GenConanfile, TestClient


def test_basic_default_modes_unknown():
    c = TestClient()
    c.save({"matrix/conanfile.py": GenConanfile("matrix"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("matrix/[*]")})
    c.run("create matrix --version=1.0")
    c.run("create engine")
    package_id = c.created_package_id("engine/1.0")

    # Using a patch version doesn't kick a engine rebuild
    c.run("create matrix --version=1.0.1")
    c.run("create engine --build=missing")
    c.assert_listed_require({"matrix/1.0.1": "Cache"})
    c.assert_listed_binary({"engine/1.0": (package_id, "Cache")})

    # same with minor version will not need rebuild
    c.run("create matrix --version=1.1.0")
    c.run("create engine --build=missing")
    c.assert_listed_require({"matrix/1.1.0": "Cache"})
    c.assert_listed_binary({"engine/1.0": (package_id, "Cache")})

    # Major will require re-build
    # TODO: Reconsider this default
    c.run("create matrix --version=2.0.0")
    c.run("create engine --build=missing")
    c.assert_listed_require({"matrix/2.0.0": "Cache"})
    c.assert_listed_binary({"engine/1.0": ("805fafebc9f7769a90dafb8c008578c6aa7f5d86", "Build")})


def test_basic_default_modes_application():
    """
    if the consumer package is a declared "package_type = "application"" recipe_revision_mode will
    be used
    """
    c = TestClient()
    c.save({"matrix/conanfile.py": GenConanfile("matrix"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("matrix/[*]")
                                                                .with_package_type("application")})
    c.run("create matrix --version=1.0")
    c.run("create engine")
    package_id = c.created_package_id("engine/1.0")

    # Using a patch version requires a rebuild
    c.run("create matrix --version=1.0.1")
    c.run("create engine --build=missing")
    c.assert_listed_require({"matrix/1.0.1": "Cache"})
    new_package_id = "efe870a1b1b4fe60e55aa6e2d17436665404370f"
    assert new_package_id != package_id
    c.assert_listed_binary({"engine/1.0": (new_package_id, "Build")})


class TestDepDefinedMode:
    def test_dep_defined(self):
        c = TestClient()
        dep = textwrap.dedent("""
            from conan import ConanFile
            class Dep(ConanFile):
                name = "dep"
                package_type = "static-library"
                package_id_embed_mode = "major_mode"
                package_id_non_embed_mode = "major_mode"
            """)
        c.save({"dep/conanfile.py": dep,
                "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requires("dep/[*]")
                                                              .with_shared_option(False)})
        c.run("create dep --version=0.1")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("66a9e6a31e63f77952fd72744d0d5da07970f42e", "Build")})
        c.run("create pkg -o pkg/*:shared=True")
        c.assert_listed_binary({"pkg/0.1": ("5a5828e18eef6a86813b01d4f5a83ea7d87d1139", "Build")})

        # using dep 0.2, still same, because dependency chose "major_mode"
        c.run("create dep --version=0.2")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("66a9e6a31e63f77952fd72744d0d5da07970f42e", "Build")})
        c.run("create pkg -o pkg/*:shared=True")
        c.assert_listed_binary({"pkg/0.1": ("5a5828e18eef6a86813b01d4f5a83ea7d87d1139", "Build")})

    def test_dep_tool_require_defined(self):
        c = TestClient()
        dep = textwrap.dedent("""
            from conan import ConanFile
            class Dep(ConanFile):
                name = "dep"
                package_type = "application"
                build_mode = "minor_mode"
            """)
        c.save({"dep/conanfile.py": dep,
                "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_tool_requires("dep/[*]")})
        c.run("create dep --version=0.1")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("fcf70699eb821f51cd4f3e228341ac4f405ad220", "Build")})

        # using dep 0.2, still same, because dependency chose "minor"
        c.run("create dep --version=0.1.1")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("fcf70699eb821f51cd4f3e228341ac4f405ad220", "Build")})

        # using dep 0.2, still same, because dependency chose "minor"
        c.run("create dep --version=0.2")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("56934f87c11792e356423e081c7cd490f3c1fbe0", "Build")})

    def test_dep_python_require_defined(self):
        c = TestClient()
        dep = textwrap.dedent("""
            from conan import ConanFile
            class Dep(ConanFile):
                name = "dep"
                package_type = "python-require"
                package_id_python_mode = "major_mode"
            """)
        c.save({"dep/conanfile.py": dep,
                "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_python_requires("dep/[*]")})
        c.run("create dep --version=0.1")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("331c17383dcdf37f79bc2b86fa55ac56afdc6fec", "Build")})

        # using dep 0.2, still same, because dependency chose "major"
        c.run("create dep --version=0.1.1")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("331c17383dcdf37f79bc2b86fa55ac56afdc6fec", "Build")})

        # using dep 0.2, still same, because dependency chose "major"
        c.run("create dep --version=0.2")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("331c17383dcdf37f79bc2b86fa55ac56afdc6fec", "Build")})
        c.run("list *:*")
        assert "dep/0.Y.Z" in c.out

        # using dep 0.2, new package_id, because dependency chose "major"
        c.run("create dep --version=1.0")
        c.run("create pkg")
        c.assert_listed_binary({"pkg/0.1": ("9b015e30b768df0217ffa2c270f60227c998e609", "Build")})
        c.run("list *:*")
        assert "dep/1.Y.Z" in c.out



import json
import platform
import textwrap
import os

from conan import conan_version
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.test.utils.env import environment_update


def test_profile_template():
    client = TestClient()
    tpl = textwrap.dedent("""\
        # This is just to check that subprocess is injected in the render and can be used
        {% set check_output = subprocess.check_output %}
        [settings]
        os = {{ {"Darwin": "Macos"}.get(platform.system(), platform.system()) }}
        build_type = {{ os.getenv("MY_BUILD_TYPE") }}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1": tpl})
    with environment_update({"MY_BUILD_TYPE": "Debug"}):
        client.run("install . -pr=profile1")

    current_os = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "os={}".format(current_os)
    assert "build_type=Debug"


def test_profile_template_variables():
    client = TestClient()
    tpl = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        [settings]
        os = {{ a }}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1": tpl})
    client.run("install . -pr=profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_import():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        {% import "profile_vars" as vars %}
        [settings]
        os = {{ vars.a }}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1": tpl1,
                 "profile_vars": tpl2})
    client.run("install . -pr=profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_import_sibling():
    # https://github.com/conan-io/conan/issues/17431
    client = TestClient()
    tpl1 = textwrap.dedent(r"""
        {% import "sub2/profile_vars" as vars %}
        [settings]
        os = {{ vars.a }}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        """)
    client.save_home({"profiles/sub1/profile1": tpl1,
                      "profiles/sub2/profile_vars": tpl2})
    client.save({"conanfile.py": GenConanfile()})
    client.run("install . -pr=sub1/profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_include():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        {% include "profile_vars" %}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        [settings]
        os = {{ a }}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1": tpl1,
                 "profile_vars": tpl2})
    client.run("install . -pr=profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_include_sibling():
    # https://github.com/conan-io/conan/issues/17431
    client = TestClient()
    tpl1 = textwrap.dedent(r"""
        {% include "sub2/profile_vars" %}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        [settings]
        os = {{ a }}
        """)
    client.save_home({"profiles/sub1/profile1": tpl1,
                      "profiles/sub2/profile_vars": tpl2})
    client.save({"conanfile.py": GenConanfile()})
    client.run("install . -pr=sub1/profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_include_from_cache():
    # https://github.com/conan-io/conan/issues/17431
    client = TestClient()
    tpl1 = textwrap.dedent(r"""
        {% include "sub2/profile_vars" %}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        [settings]
        os = {{ a }}
        """)
    client.save_home({"profiles/sub2/profile_vars": tpl2})
    client.save({"conanfile.py": GenConanfile(),
                 "sub1/profile1": tpl1})
    client.run("install . -pr=sub1/profile1")
    assert "os=FreeBSD" in client.out


def test_profile_template_profile_dir():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        [conf]
        user.toolchain:mydir = {{ os.path.join(profile_dir, "toolchain.cmake") }}
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import load
        class Pkg(ConanFile):
            def generate(self):
                content = load(self, self.conf.get("user.toolchain:mydir"))
                self.output.info("CONTENT: {}".format(content))
        """)
    client.save({"conanfile.py": conanfile,
                 "anysubfolder/profile1": tpl1,
                 "anysubfolder/toolchain.cmake": "MyToolchainCMake!!!"})
    client.run("install . -pr=anysubfolder/profile1")
    assert "conanfile.py: CONTENT: MyToolchainCMake!!!" in client.out


def test_profile_conf_backslash():
    # https://github.com/conan-io/conan/issues/15726
    c = TestClient()
    profile = textwrap.dedent(r"""
        [conf]
        user.team:myconf = "hello\test"
        """)
    c.save({"profile": profile})
    c.run("profile show -pr=profile")
    assert r"hello\test" in c.out


def test_profile_version():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        [options]
        *:myoption={{conan_version}}
        *:myoption2={{conan_version<13 and conan_version>1.0}}
        """)

    client.save({"conanfile.py": GenConanfile(),
                 "profile1.jinja": tpl1})
    client.run("install . -pr=profile1.jinja")
    assert f"*:myoption={conan_version}" in client.out
    assert "*:myoption2=True" in client.out


def test_profile_template_profile_name():
    """
        The property profile_name should be parsed as the profile file name when rendering profiles
    """
    client = TestClient()
    tpl1 = textwrap.dedent("""
        [conf]
        user.profile:name = {{ profile_name }}
        """)
    tpl2 = textwrap.dedent("""
        include(default)
        [conf]
        user.profile:name = {{ profile_name }}
        """)
    default = textwrap.dedent("""
        [settings]
        os=Windows
        [conf]
        user.profile:name = {{ profile_name }}
        """)

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def configure(self):
                self.output.info("PROFILE NAME: {}".format(self.conf.get("user.profile:name")))
        """)
    client.save({"conanfile.py": conanfile,
                 "profile_folder/foobar": tpl1,
                 "another_folder/foo.profile": tpl1,
                 "include_folder/include_default": tpl2,
                 os.path.join(client.paths.profiles_path, "baz"): tpl1,
                 os.path.join(client.paths.profiles_path, "default"): default})

    # show only file name as profile name
    client.run("install . -pr=profile_folder/foobar")
    assert "conanfile.py: PROFILE NAME: foobar" in client.out

    # profile_name should include file extension
    client.run("install . -pr=another_folder/foo.profile")
    assert "conanfile.py: PROFILE NAME: foo.profile" in client.out

    # default profile should compute profile_name by default too
    client.run("install . -pr=default")
    assert "conanfile.py: PROFILE NAME: default" in client.out

    # profile names should show only their names
    client.run("install . -pr=baz")
    assert "conanfile.py: PROFILE NAME: baz" in client.out

    # included profiles should respect the inherited profile name
    client.run("install . -pr=include_folder/include_default")
    assert "conanfile.py: PROFILE NAME: include_default" in client.out


class TestProfileDetectAPI:
    def test_profile_detect_os_arch(self):
        """ testing OS & ARCH just to test the UX and interface
        """
        client = TestClient()
        tpl1 = textwrap.dedent("""
            [settings]
            os={{detect_api.detect_os()}}
            arch={{detect_api.detect_arch()}}
            """)

        client.save({"profile1": tpl1})
        client.run("profile show -pr=profile1 --context=host")
        pr = client.get_default_host_profile()
        the_os = pr.settings['os']
        arch = pr.settings['arch']
        expected = textwrap.dedent(f"""\
            [settings]
            arch={arch}
            os={the_os}
            """)
        assert expected in client.out

    def test_profile_detect_compiler_missing_error(self):
        client = TestClient(light=True)
        tpl1 = textwrap.dedent("""
                    {% set compiler, version, compiler_exe = detect_api.detect_clang_compiler(compiler_exe="not-existing-compiler") %}
                    {% set version = detect_api.default_compiler_version(compiler, version) %}
                    """)

        client.save({"profile1": tpl1})
        client.run("profile show -pr=profile1", assert_error=True)
        assert "No version provided to 'detect_api.default_compiler_version()' for None compiler" in client.out


def test_profile_jinja_error():
    c = TestClient(light=True)
    profile = textwrap.dedent("""
        {% set arr = [
            {# comment that triggers the error #}
            'smth'
        ] %}""")
    c.save({"profile1": profile,
            "profile2": "include(profile1)",
            "profile3": "include(profile2)",
            "conanfile.txt": ""})
    c.run("profile show -pr=profile1", assert_error=True)
    assert "ERROR: Error while rendering the profile template file" in c.out
    c.run("profile show -pr=profile2", assert_error=True)
    assert "ERROR: Error reading 'profile2' profile: Error while " \
           "rendering the profile template file " in c.out
    assert "unexpected char '#' at 21" in c.out
    c.run("profile show -pr=profile3", assert_error=True)
    assert "ERROR: Error reading 'profile3' profile: Error reading 'profile2' profile: Error while " \
           "rendering the profile template file " in c.out
    assert "unexpected char '#' at 21" in c.out

    # Also install messages
    c.run("install . -pr=profile1", assert_error=True)
    assert "ERROR: Error while rendering the profile template file" in c.out
    assert "unexpected char '#' at 21" in c.out

    c.run("install . -pr=profile2", assert_error=True)
    assert "ERROR: Error reading 'profile2' profile: Error while " \
           "rendering the profile template file " in c.out
    assert "unexpected char '#' at 21" in c.out
    c.run("install . -pr=profile3", assert_error=True)
    assert "ERROR: Error reading 'profile3' profile: Error reading 'profile2' profile: Error while " \
           "rendering the profile template file " in c.out
    assert "unexpected char '#' at 21" in c.out


def test_profile_macro_per_package():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        {% macro set_windows_settings(lib) %}
        {{lib}}/*:os=Windows
        {{lib}}/*:arch=x86
        {% endmacro %}
        {% macro set_windows_conf(lib) %}
        {{lib}}/*:user.conf:key = 2
        {% endmacro %}

        [settings]
        os = Linux
        {{set_windows_settings("mypkg")}}
        [conf]
        user.conf:key = 1
        {{set_windows_conf("mypkg")}}
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "mypkg"
            version = "0.1"
            settings = "os", "arch"
            def generate(self):
                value = self.conf.get("user.conf:key")
                self.output.info(f"os={self.settings.os}!!")
                self.output.info(f"arch={self.settings.arch}!!")
                self.output.info(f"user.conf:key={value}!!!!")
        """)
    client.save({"conanfile.py": conanfile,
                 "profile1": tpl1})
    client.run("install . -pr=profile1")
    assert "conanfile.py (mypkg/0.1): user.conf:key=2!!!!" in client.out
    assert "conanfile.py (mypkg/0.1): os=Windows!!" in client.out
    assert "conanfile.py (mypkg/0.1): arch=x86!!" in client.out


def test_profile_jinja_context():
    """
        Test the ``context=build/host`` injection
    """
    c = TestClient()
    tpl1 = textwrap.dedent("""
        [conf]
        {% if context == "host" %}
        user.profile:common = myhost
        {% elif context == "build" %}
        user.profile:common = mybuild
        {% else %}
        user.profile:common= nocontext
        [settings]
        os = Linux
        {% endif %}
        """)
    tpl2 = textwrap.dedent("""
        include(common)
        [conf]
        {% if context == "host" %}
        user.profile:base = myhost
        {% elif context == "build" %}
        user.profile:base = mybuild
        {% else %}
        user.profile:base = nocontext
        [settings]
        arch = x86
        {% endif %}
        """)

    c.save({"common": tpl1,
            "base": tpl2})

    c.run("profile show -pr:a=base --format=json")
    profiles = json.loads(c.stdout)
    assert profiles["host"]["conf"] == {"user.profile:common": "myhost",
                                        "user.profile:base": "myhost"}
    assert profiles["build"]["conf"] == {"user.profile:common": "mybuild",
                                         "user.profile:base": "mybuild"}

    # Now lets test when profile is neither build/host, like when used in ``conan list``
    c.save({"conanfile.py": GenConanfile("pkg", "0.1").with_settings("os", "arch")})
    c.run("create . -s os=Linux -s arch=x86")
    c.run("create . -s os=Windows -s arch=armv8")

    # This will pick the Linux+x86 package binary
    c.run("list *:* --filter-profile=base")
    assert "os: Windows" not in c.out
    assert "arch: armv8" not in c.out
    assert "os: Linux" in c.out
    assert "arch: x86" in c.out



import os
from unittest import mock

from conan import __version__
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


class TestRequiredVersion:

    @mock.patch("conan.__version__", "1.26.0")
    def test_wrong_version(self):
        required_version = "1.23.0"
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version={required_version}"})
        client.run("help", assert_error=True)
        assert ("Current Conan version (1.26.0) does not satisfy the defined "
                f"one ({required_version})") in client.out

    @mock.patch("conan.__version__", "1.22.0")
    def test_exact_version(self):
        required_version = "1.22.0"
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version={required_version}"})
        client.run("--help")

    @mock.patch("conan.__version__", "2.1.0")
    def test_lesser_version(self):
        required_version = "<3.0"
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version={required_version}"})
        client.run("--help")

    @mock.patch("conan.__version__", "1.0.0")
    def test_greater_version(self):
        required_version = ">0.1.0"
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version={required_version}"})
        client.run("--help")

    @mock.patch("conan.__version__", "1.0.0")
    def test_greater_version_cc_override(self):
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version=>0.1.0"})
        client.run("config home -cc core:required_conan_version=>2.0", assert_error=True)
        assert f"Current Conan version (1.0.0) does not satisfy the defined one (>2.0)" in client.out

    @mock.patch("conan.__version__", "1.0.0")
    def test_version_check_before_hooks(self):
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core:required_conan_version=>2.0",
                          # The hook shouldn't matter, version check happens earlier
                          "extensions/hooks/hook_trim.py": "some error here"})
        client.run("config home", assert_error=True)
        assert f"Current Conan version (1.0.0) does not satisfy the defined one (>2.0)" in client.out

    def test_bad_format(self):
        required_version = "1.0.0.0-foobar"
        cache_folder = temp_folder()
        save(os.path.join(cache_folder, "global.conf"),
             f"core:required_conan_version={required_version}")
        c = TestClient(cache_folder, light=True)
        c.run("version", assert_error=True)
        assert (f"Current Conan version ({__version__}) does not satisfy "
                f"the defined one ({required_version})") in c.out



import os

from conan.api.model.list import PackagesList
from conan.api.output import ConanOutput
from conan.api.model import PkgReference
from conan.api.model import RecipeReference


class IntegrityChecker:
    """
    Check:
        - Performs a corruption integrity check in the cache. This is done by loading the existing
        conanmanifest.txt and comparing against a computed conanmanifest.txt. It
        doesn't address someone tampering with the conanmanifest.txt, just accidental
        modifying of a package contents, like if some file has been added after computing the
        manifest.
        This is to be done over the package contents, not the compressed conan_package.tgz
        artifacts
    """
    def __init__(self, cache):
        self._cache = cache

    def check(self, pkg_list) -> PackagesList:
        corrupted_pkglist = PackagesList()
        for ref, packages in pkg_list.items():
            # Check if any of the packages are corrupted
            if self._recipe_corrupted(ref):
                # If the recipe is corrupted, all its packages are considered corrupted
                corrupted_pkglist.add_ref(ref)
            else:
                # Do not check any binary if the recipe is corrupted
                for pref in packages:
                    if self._package_corrupted(pref):
                        corrupted_pkglist.add_ref(ref)
                        # Cannot add package reference without having the recipe reference already added
                        corrupted_pkglist.add_pref(pref)
        return corrupted_pkglist

    def _recipe_corrupted(self, ref: RecipeReference):
        layout = self._cache.recipe_layout(ref)
        output = ConanOutput(scope=f"{ref.repr_notime()}")
        try:
            read_manifest, expected_manifest = layout.recipe_manifests()
        except FileNotFoundError:
            output.error("Manifest missing", error_type="exception")
            return True
        # Filter exports_sources from read manifest if there are no exports_sources locally
        # This happens when recipe is downloaded without sources (not built from source)
        export_sources_folder = layout.export_sources()
        if not os.path.exists(export_sources_folder):
            read_manifest.file_sums = {k: v for k, v in read_manifest.file_sums.items()
                                       if not k.startswith("export_source")}

        if read_manifest != expected_manifest:
            output_lines = ["", "Manifest mismatch", f"    Folder: {layout.package()}"]
            diff = read_manifest.difference(expected_manifest)
            for fname, (h1, h2) in diff.items():
                output_lines.append(f"        {fname} (manifest: {h1}, file: {h2})")
            output.error("\n".join(output_lines), error_type="exception")
            return True
        output.info("Integrity check: ok")

    def _package_corrupted(self, ref: PkgReference):
        layout = self._cache.pkg_layout(ref)
        output = ConanOutput(scope=f"{ref.repr_notime()}")
        try:
            read_manifest, expected_manifest = layout.package_manifests()
        except FileNotFoundError:
            output.error("Manifest missing", error_type="exception")
            return True

        if read_manifest != expected_manifest:
            output_lines = ["", "Manifest mismatch", f"    Folder: {layout.package()}"]
            diff = read_manifest.difference(expected_manifest)
            for fname, (h1, h2) in diff.items():
                output_lines.append(f"        {fname} (manifest: {h1}, file: {h2})")
            output.error("\n".join(output_lines), error_type="exception")
            return True
        output.info("Integrity check: ok")



import pytest

from conan.cli.exit_codes import ERROR_INVALID_CONFIGURATION
from conan.test.utils.tools import TestClient


class TestInvalidConfiguration:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.client = TestClient()
        self.client.save({"conanfile.py": """
from conan import ConanFile
from conan.errors import ConanInvalidConfiguration

class MyPkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def configure(self):
        if self.settings.compiler.version == "190":
            raise ConanInvalidConfiguration("user says that compiler.version=12 is invalid")

    """})
        settings = "-s os=Windows -s compiler=msvc -s compiler.version={ver} "\
                   "-s compiler.runtime=dynamic"
        self.settings_msvc15 = settings.format(ver="192")
        self.settings_msvc12 = settings.format(ver="190")

    def test_install_method(self):
        self.client.run("install . %s" % self.settings_msvc15)

        error = self.client.run("install . %s" % self.settings_msvc12, assert_error=True)
        assert error == ERROR_INVALID_CONFIGURATION
        assert "Invalid configuration: user says that compiler.version=12 is invalid" \
                      in self.client.out

    def test_info_method(self):
        self.client.run("graph info . %s" % self.settings_msvc15)

        error = self.client.run("graph info . %s" % self.settings_msvc12,
                                assert_error=True)
        assert error == ERROR_INVALID_CONFIGURATION
        assert "ERROR: conanfile.py: Invalid configuration: " \
                      "user says that compiler.version=12 is invalid" in self.client.out

    def test_create_method(self):
        self.client.run("create . --name=name --version=ver --user=jgsogo --channel=test %s" % self.settings_msvc15)

        error = self.client.run("create . --name=name --version=ver --user=jgsogo --channel=test %s" % self.settings_msvc12,
                                assert_error=True)
        assert error == ERROR_INVALID_CONFIGURATION
        assert "name/ver@jgsogo/test: Invalid configuration: user says that " \
                      "compiler.version=12 is invalid" in self.client.out

    def test_as_requirement(self):
        self.client.run("create . --name=name --version=ver %s" % self.settings_msvc15)
        self.client.save({"other/conanfile.py": """
from conan import ConanFile
from conan.errors import ConanInvalidConfiguration

class MyPkg(ConanFile):
    requires = "name/ver"
    settings = "os", "compiler", "build_type", "arch"
    """})
        self.client.run("create other/ --name=other --version=1.0 %s" % self.settings_msvc15)

        error = self.client.run("create other/ --name=other --version=1.0 %s" % self.settings_msvc12,
                                assert_error=True)
        assert error == ERROR_INVALID_CONFIGURATION
        assert "name/ver: Invalid configuration: user says that " \
                      "compiler.version=12 is invalid" in self.client.out



import textwrap
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save_files
from conan.test.utils.test_files import temp_folder


def _create_py_hello_world(folder):
    setup_py = textwrap.dedent("""
        from setuptools import setup, find_packages

        setup(
            name='hello',
            version='0.1.0',
            packages=find_packages(include=['hello', 'hello.*']),
            entry_points={'console_scripts': ['hello-world = hello:hello']}
        )
        """)
    hello_py = textwrap.dedent("""
        def hello():
            print("Hello Test World!")
        """)

    save_files(folder, {"setup.py": setup_py, "hello/__init__.py": hello_py})


def test_empty_pipenv():
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.system import PipEnv

        class PipPackage(ConanFile):

            def generate(self):
                PipEnv(self).generate()

            def build(self):
                self.run("python -m pip list")
        """)

    c = TestClient(path_with_spaces=False)
    c.save({"conanfile.py": conanfile})
    c.run("build")
    # Test that some Conan common deps are not in this pip list
    assert "requests" not in c.out
    assert "colorama" not in c.out
    assert "Jinja2" not in c.out
    assert "PyJWT" not in c.out


def test_build_pip_manager():

    pip_package_folder = temp_folder(path_with_spaces=True)
    _create_py_hello_world(pip_package_folder)
    pip_package_folder = pip_package_folder.replace('\\', '/')

    conanfile_pip = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.system import PipEnv
        from conan.tools.layout import basic_layout
        import platform
        import os


        class PipPackage(ConanFile):
            name = "pip_hello_test"
            version = "0.1"

            def layout(self):
                basic_layout(self)

            def generate(self):
                PipEnv(self).install(["{pip_package_folder}"])
                PipEnv(self).generate()

            def build(self):
                self.run("hello-world")
        """)

    client = TestClient(path_with_spaces=False)
    # FIXME: the python shebang inside vitual env packages fails when using path_with_spaces
    client.save({"pip/conanfile.py": conanfile_pip})
    client.run("build pip/conanfile.py")

    assert "RUN: hello-world" in client.out
    assert "Hello Test World!" in client.out


def test_create_pip_manager():

    pip_package_folder = temp_folder(path_with_spaces=True)
    _create_py_hello_world(pip_package_folder)
    pip_package_folder = pip_package_folder.replace('\\', '/')

    conanfile_pip = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.system import PipEnv
        from conan.tools.layout import basic_layout
        import platform
        import os


        class PipPackage(ConanFile):
            name = "pip_hello_test"
            version = "0.1"
            build_policy = "missing"
            upload_policy = "skip"

            def layout(self):
                basic_layout(self)

            def finalize(self):
                PipEnv(self, self.package_folder).install(["{pip_package_folder}"])

            def package_info(self):
                python_env_bin = PipEnv(self, self.package_folder).bin_dir
                self.buildenv_info.prepend_path("PATH", python_env_bin)
        """)

    conanfile = textwrap.dedent("""
        from conan import ConanFile


        class Recipe(ConanFile):
            name = "pip_test"
            version = "0.1"

            def requirements(self):
                self.tool_requires("pip_hello_test/0.1")

            def build(self):
                self.run("hello-world")
        """)

    client = TestClient(path_with_spaces=False)
    # FIXME: the python shebang inside vitual env packages fails when using path_with_spaces
    client.save({"pip/conanfile.py": conanfile_pip, "consumer/conanfile.py": conanfile})
    client.run("create pip/conanfile.py --version=0.1")
    client.run("build consumer/conanfile.py")

    assert "RUN: hello-world" in client.out
    assert "Hello Test World!" in client.out



import pytest

from conan.internal.util.files import copytree_compat


@pytest.fixture
def source_dest_folders(tmp_path):
    source_folder = tmp_path / "source"
    dest_folder = tmp_path / "dest"
    source_folder.mkdir()
    dest_folder.mkdir()
    test_file = source_folder / "test_file.txt"
    test_file.write_text("Test content")
    return source_folder, dest_folder


def test_copytree_compat(source_dest_folders):
    source_folder, dest_folder = source_dest_folders
    copytree_compat(str(source_folder), str(dest_folder))
    assert (dest_folder / "test_file.txt").exists()



import json
import time

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestLRU:
    def test_error_lru_remote(self):
        c = TestClient(default_server_user=True)
        c.run("list * --lru=1s -r=default", assert_error=True)
        assert "'--lru' cannot be used in remotes, only in cache" in c.out

    @pytest.mark.parametrize("method", ["list", "remove"])
    def test_cache_clean_lru(self, method):
        c = TestClient()
        c.save({"conanfile.py": GenConanfile()})
        c.run("create . --name=pkg --version=0.1")
        c.run("create . --name=dep --version=0.2")

        time.sleep(2)
        # This should update the LRU
        c.run("install --requires=pkg/0.1")
        # Removing recipes (+ its binaries) that recipes haven't been used
        if method == "list":
            c.run("list *#* --lru=1s --format=json", redirect_stdout="old.json")
            c.run("remove --list=old.json -c")
        else:
            c.run("remove * --lru=1s -c")
        # Do the checks
        c.run("list *:*#*")
        assert "pkg" in c.out
        assert "da39a3ee5e6b4b0d3255bfef95601890afd80709" in c.out
        assert "dep" not in c.out

        time.sleep(2)
        # This should update the LRU of the recipe only
        c.run("graph info --requires=pkg/0.1")
        # IMPORTANT: Note the pattern is NOT the same as the equivalent for 'conan remove'
        if method == "list":
            c.run("list *#*:*#* --lru=1s --format=json", redirect_stdout="old.json")
            c.run("remove --list=old.json -c")
        else:
            c.run("remove *:* --lru=1s -c")

        # Check the binary has been removed, but the recipe is still there
        c.run("list *:*")

        assert "pkg" in c.out
        assert "da39a3ee5e6b4b0d3255bfef95601890afd80709" not in c.out
        assert "dep" not in c.out

    def test_lru_just_created(self):
        c = TestClient()
        c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
        c.run("create .")
        c.run("list * --lru=1d", assert_error=True)
        assert "'--lru' must be used with recipe revision pattern" in c.out
        c.run("list *#* --lru=1d")
        assert "pkg/0.1" not in c.out
        # What if no revision, but package id is passed
        c.run("list pkg/0.1:* --lru=1d", assert_error=True)
        assert "'--lru' must be used with package revision pattern" in c.out
        # Doesn't fail, but packages is empty
        c.run("list pkg/0.1:*#* --lru=1d --format=json")
        pkgs = json.loads(c.stdout)
        rev = pkgs["Local Cache"]["pkg/0.1"]["revisions"]["485dad6cb11e2fa99d9afbe44a57a164"]
        assert rev["packages"] == {}

    def test_update_lru_when_used_as_dependency(self):
        """Show that using a recipe as a dependency will update its LRU"""
        c = TestClient()
        c.save({"dep/conanfile.py": GenConanfile("dep", "1.0"),
                "conanfile.py": GenConanfile("app", "1.0").with_require("dep/1.0")})

        c.run("create dep")
        time.sleep(2)
        c.run("create .")
        # Dep is not removed because its lru was updated as part of the above create
        c.run("remove * --lru=1s -c -f=json", redirect_stdout="removed.json")
        removed = json.loads(c.load("removed.json"))
        assert len(removed["Local Cache"]) == 0



import platform
import textwrap
import os

import pytest

from conan.test.assets.autotools import gen_makefile_am, gen_configure_ac
from conan.test.assets.genconanfile import GenConanfile
from conan.test.assets.sources import gen_function_cpp
from test.conftest import tools_locations
from test.functional.utils import check_exe_run, check_vs_runtime
from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
@pytest.mark.tool("msys2")
def test_autotools_bash_complete():
    client = TestClient(path_with_spaces=False)
    profile_win = textwrap.dedent(f"""
        include(default)
        [conf]
        tools.microsoft.bash:subsystem=msys2
        tools.microsoft.bash:path=bash
        """)

    main = gen_function_cpp(name="main")
    # The autotools support for "cl" compiler (VS) is very limited, linking with deps doesn't
    # work but building a simple app do
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import Autotools

        class TestConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp"
            generators = "AutotoolsToolchain"
            win_bash = True

            def build(self):
                # These commands will run in bash activating first the vcvars and
                # then inside the bash activating the
                self.run("aclocal")
                self.run("autoconf")
                self.run("automake --add-missing --foreign")
                autotools = Autotools(self)
                autotools.configure()
                autotools.make()
                autotools.install()
        """)

    client.save({"conanfile.py": conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main,
                 "profile_win": profile_win})
    client.run("build . -pr=profile_win")
    client.run_command("main.exe")
    check_exe_run(client.out, "main", "msvc", None, "Release", "x86_64", None)

    bat_contents = client.load("conanbuild.bat")
    assert "conanvcvars.bat" in bat_contents


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
@pytest.mark.tool("clang", "18")
@pytest.mark.tool("msys2")
@pytest.mark.parametrize("frontend", ("clang", "clang-cl"))
@pytest.mark.parametrize("runtime", ("static", "dynamic"))
@pytest.mark.parametrize("build_type", ("Debug", "Release"))
def test_autotools_bash_complete_clang(frontend, runtime, build_type):
    client = TestClient(path_with_spaces=False)
    # Problem is that msys2 also has clang in the path, so we need to make it explicit
    clangpath = tools_locations["clang"]["18"]["path"]["Windows"]
    # compilers
    c, cpp = ("clang", "clang++") if frontend == "clang" else ("clang-cl", "clang-cl")
    comps = f'{{"cpp":"{clangpath}/{cpp}", "c":"{clangpath}/{c}", "rc":"{clangpath}/{c}"}}'
    profile_win = textwrap.dedent(f"""
        [settings]
        os=Windows
        arch=x86_64
        build_type={build_type}
        compiler=clang
        compiler.version=18
        compiler.cppstd=14
        compiler.runtime_version=v144
        compiler.runtime={runtime}

        [conf]
        tools.build:compiler_executables={comps}
        tools.microsoft.bash:subsystem=msys2
        tools.microsoft.bash:path=bash
        tools.compilation:verbosity=verbose
        """)

    main = gen_function_cpp(name="main")
    # The autotools support for "cl" compiler (VS) is very limited, linking with deps doesn't
    # work but building a simple app do
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import Autotools

        class TestConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp"
            generators = "AutotoolsToolchain"
            win_bash = True

            def build(self):
                # These commands will run in bash activating first the vcvars and
                # then inside the bash activating the
                self.run("aclocal")
                self.run("autoconf")
                self.run("automake --add-missing --foreign")
                autotools = Autotools(self)
                autotools.configure()
                autotools.make()
                autotools.install()
        """)

    client.save({"conanfile.py": conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main,
                 "profile_win": profile_win})
    client.run("build . -pr=profile_win")
    client.run_command("main.exe")
    assert "__GNUC__" not in client.out
    assert "main __clang_major__18" in client.out
    check_exe_run(client.out, "main", "clang", None, build_type, "x86_64", None)

    bat_contents = client.load("conanbuild.bat")
    assert "conanvcvars.bat" in bat_contents

    static_runtime = runtime == "static"
    check_vs_runtime("main.exe", client, "17", build_type=build_type, static_runtime=static_runtime)


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_add_msys2_path_automatically():
    """ Check that commands like ar, autoconf, etc, that are in the /usr/bin folder together
    with the bash.exe, can be automaticallly used when running in windows bash, without user
    extra addition to [buildenv] of that msys64/usr/bin path

    # https://github.com/conan-io/conan/issues/12110
    """
    client = TestClient(path_with_spaces=False)
    bash_path = None
    try:
        bash_path = tools_locations["msys2"]["system"]["path"]["Windows"] + "/bash.exe"
    except KeyError:
        pytest.skip("msys2 path not defined")

    client.save_home({"global.conf": textwrap.dedent("""
            tools.microsoft.bash:subsystem=msys2
            tools.microsoft.bash:path={}
            """.format(bash_path))})

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class HelloConan(ConanFile):
            name = "hello"
            version = "0.1"

            win_bash = True

            def build(self):
                self.run("ar -h")
                """)

    client.save({"conanfile.py": conanfile})
    client.run("create .")
    assert "ar.exe" in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_conf_inherited_in_test_package():
    client = TestClient()
    bash_path = None
    try:
        bash_path = tools_locations["msys2"]["system"]["path"]["Windows"] + "/bash.exe"
    except KeyError:
        pytest.skip("msys2 path not defined")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):
            name="msys2"
            version="1.0"

            def package_info(self):
                self.conf_info.define("tools.microsoft.bash:subsystem", "msys2")
                self.conf_info.define("tools.microsoft.bash:path", r"{}")
    """.format(bash_path))
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    conanfile = GenConanfile("consumer", "1.0")
    test_package = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):
            name="test"
            version="1.0"
            win_bash = True

            def build_requirements(self):
                self.tool_requires(self.tested_reference_str)
                self.tool_requires("msys2/1.0")

            def build(self):
                self.output.warning(self.conf.get("tools.microsoft.bash:subsystem"))
                self.run("aclocal --version")

            def test(self):
                pass
        """)
    client.save({"conanfile.py": conanfile, "test_package/conanfile.py": test_package})
    client.run("create . -s:b os=Windows -s:h os=Windows")
    assert "are needed to run commands in a Windows subsystem" not in client.out
    assert "aclocal (GNU automake)" in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
@pytest.mark.tool("msys2")
def test_msys2_and_msbuild():
    """ Check that msbuild can be executed in msys2 environment

    # https://github.com/conan-io/conan/issues/15627
    """
    client = TestClient(path_with_spaces=False)
    profile_win = textwrap.dedent(f"""
        include(default)
        [conf]
        tools.microsoft.bash:subsystem=msys2
        tools.microsoft.bash:path=bash
        """)

    main = gen_function_cpp(name="main")
    # The autotools support for "cl" compiler (VS) is very limited, linking with deps doesn't
    # work but building a simple app do
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import Autotools
        from conan.tools.microsoft import MSBuild

        class TestConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp", "MyProject.vcxproj"
            generators = "AutotoolsToolchain"
            win_bash = True

            def build(self):
                # These commands will run in bash activating first the vcvars and
                # then inside the bash activating the
                self.run("aclocal")
                self.run("autoconf")
                self.run("automake --add-missing --foreign")
                autotools = Autotools(self)
                autotools.configure()
                autotools.make()
                autotools.install()
                msbuild = MSBuild(self)
                msbuild.build("MyProject.vcxproj")
        """)

    # A minimal project is sufficient - here just copy the application file to another directory
    my_vcxproj = r"""<?xml version="1.0" encoding="utf-8"?>
        <Project DefaultTargets="Build" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
        <ItemGroup Label="ProjectConfigurations">
        <ProjectConfiguration Include="Debug|Win32">
          <Configuration>Debug</Configuration>
          <Platform>Win32</Platform>
        </ProjectConfiguration>
        <ProjectConfiguration Include="Release|Win32">
          <Configuration>Release</Configuration>
          <Platform>Win32</Platform>
        </ProjectConfiguration>
        <ProjectConfiguration Include="Debug|x64">
          <Configuration>Debug</Configuration>
          <Platform>x64</Platform>
        </ProjectConfiguration>
        <ProjectConfiguration Include="Release|x64">
          <Configuration>Release</Configuration>
          <Platform>x64</Platform>
        </ProjectConfiguration>
      </ItemGroup>
      <PropertyGroup Label="Globals">
        <ProjectGuid>{B58316C0-C78A-4E9B-AE8F-5D6368CE3840}</ProjectGuid>
        <Keyword>Win32Proj</Keyword>
      </PropertyGroup>
      <Import Project="$(VCTargetsPath)\Microsoft.Cpp.Default.props" />
      <PropertyGroup>
        <ConfigurationType>Application</ConfigurationType>
        <PlatformToolset>v141</PlatformToolset>
      </PropertyGroup>
      <Import Project="$(VCTargetsPath)\Microsoft.Cpp.props" />
      <ImportGroup Label="PropertySheets">
        <Import Project="$(UserRootDir)\Microsoft.Cpp.$(Platform).user.props" Condition="exists('$(UserRootDir)\Microsoft.Cpp.$(Platform).user.props')" />
      </ImportGroup>
      <PropertyGroup>
        <OutDir>$(ProjectDir)msbuild_out</OutDir>
      </PropertyGroup>
      <ItemDefinitionGroup>
      </ItemDefinitionGroup>
      <ItemGroup>
        <Content Include="$(ProjectDir)main.exe">
          <CopyToOutputDirectory>PreserveNewest</CopyToOutputDirectory>
        </Content>
      </ItemGroup>
      <Import Project="$(VCTargetsPath)\Microsoft.Cpp.targets" />
    </Project>
    """

    client.save({"conanfile.py": conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main,
                 "profile_win": profile_win,
                 "MyProject.vcxproj": my_vcxproj})
    client.run("build . -pr=profile_win")
    # Run application in msbuild output directory
    client.run_command(os.path.join("msbuild_out", "main.exe"))
    check_exe_run(client.out, "main", "msvc", None, "Release", "x86_64", None)

    bat_contents = client.load("conanbuild.bat")
    assert "conanvcvars.bat" in bat_contents



import textwrap

from conan.test.utils.tools import TestClient


def test_disable_settings_assignment():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"
            def generate(self):
                self.settings.os = "FreeBSD"
            """)
    c.save({"conanfile.py": conanfile})
    c.run("install .", assert_error=True)
    assert "Tried to define 'os' setting inside recipe" in c.out
